<?php
/** 
* Links administration
*
* Controller for displaying, modifying and deleting links.
* 
* @package      ezpx 
* @subpackage   Admin
* @author       Peter Zimon 
*/ 
class Links extends Controller {

  var $header_data;

  function Links()
  {
    parent::Controller();
    
    $this->load->database();
    $this->load->model('Ezpx');
    
    $libraries = array('session', 'form_validation');
    $this->header_data['header_title'] = $this->lang->line('links');
    $this->load->library($libraries);
    
    $helpers = array('form');
    $this->load->helper($helpers);
        
    if ($this->session->userdata('logged_in') != TRUE) 
    {
      redirect('admin/login');
    }
  }
  
  
  /** 
  * Default function
  * 
  * Display all links with pagination
  * 
  * @access public
  */ 
  function index()
  {
    $links = $this->Ezpx->get_all_links();
    $data['links'] = $links->result_array();
    
    // Call views
    $this->load->view('admin/header', $this->header_data);
    $this->load->view('admin/links', $data);
    $this->load->view('admin/footer');
  }
  
  
  /** 
  * Insert new link in database
  *
  * Display new link form, validate post data and add new link to database
  * 
  * @access public
  */ 
  function add()
  {
    $this->form_validation->set_rules('title', $this->lang->line('title'), 'required');
    $this->form_validation->set_rules('url', $this->lang->line('url'), 'required');
    $this->form_validation->set_rules('body', '', '');
    
    if ($this->form_validation->run() == TRUE && ! $this->config->item('demo_mode'))
    {
      // Add page
      if ($this->Ezpx->insert_link($this->input))
      {
        redirect('admin/links');
      }
      else
      {
        $error['error'] = 'An error occured during database insert.';
        
        // Call views
        $this->load->view('admin/header', $this->header_data);
        $this->load->view('admin/error', $error);
        $this->load->view('admin/footer');
      }
    }
    else
    {
      $data['title'] = '';
      $data['url'] = 'http://';
      $data['body'] = '';
      $data['action'] = 'admin/links/add';
    
      // Call views
      $this->load->view('admin/header', $this->header_data);
      $this->load->view('admin/links_maintain', $data);
      $this->load->view('admin/footer');
    }
  }
  
  
  /** 
  * Modify links
  *
  * Display link form with pre-filled values of the modified link. Validate post
  * data and call update database.
  * 
  * @access public
  */ 
  function modify()
  {
    $id = $this->uri->segment(4, 0);
    if ( ! $id) return;
    
    $link = $this->Ezpx->get_link($id);
    $link = $link->result_array();
    $link = $link[0];
  
    $this->form_validation->set_rules('title', $this->lang->line('title'), 'required');
    $this->form_validation->set_rules('url', $this->lang->line('url'), 'required');
    $this->form_validation->set_rules('body', '', '');
    
    if ($this->form_validation->run() == TRUE && ! $this->config->item('demo_mode'))
    {
      // Add page
      if ($this->Ezpx->update_link($id, $this->input))
      {
        redirect('admin/links');
      }
      else
      {
        $error['error'] = 'An error occured during database insert.';
        
        // Call views
        $this->load->view('admin/header', $this->header_data);
        $this->load->view('admin/error', $error);
        $this->load->view('admin/footer');
      }
    }
    else
    {
      $data = $link;
      $data['action'] = 'admin/links/modify/' . $id;
    
      // Call views
      $this->load->view('admin/header', $this->header_data);
      $this->load->view('admin/links_maintain', $data);
      $this->load->view('admin/footer');
    }
  }
  
  
  /**
  * Delete link
  *
  * @access  public
  */
  function delete()
  {
    // Check demo mode
    if ($this->config->item('demo_mode'))
    {
      redirect('admin/links');
    }
  
    $id = $this->uri->segment(4);
    $error['error'] = '';
    
    if ( ! $this->Ezpx->delete_link($id))
    {
      $error['error'] = 'An error occured during deleting link.';
    }
    
    // If error occured -> display it
    if ($error['error'])
    {
      $this->load->view('admin/header', $this->header_data);
      $this->load->view('admin/error', $error);
      $this->load->view('admin/footer');
    }
    else
    {
      redirect('admin/links');
    }    
  }

}
?>