<?php
/**
* Admin Login class
* 
* Login to the administration area
* 
* @package      ezpx
* @subpackage   Admin
* @author       Peter Zimon
*/
class Login extends Controller {
  
  function Login()
  {
    parent::Controller();
    
    // Load database and models
    $this->load->database();
    $this->load->model('Ezpx');
    $this->load->model('Installer');
    
    // Check installation
    if ( ! $this->Installer->check_install())
    {
      redirect('install');
    }
    
    // Load libraries
    $libraries = array('session', 'form_validation');
    $this->load->library($libraries);
    
    // Load helpers
    $helpers = array('form', 'url');
    $this->load->helper($helpers);
  } 
  
  
  /**
  * Default method
  * 
  * Decides whether the login screen or the admin main screen 
  * should be displayed.
  *
  * @access   public
  */
  function index()
  { 
    // Check whether user is already logged in
    if ($this->session->userdata('logged_in') == TRUE)
    {
      redirect('admin/main');
    }
    else
    {
      $this->form_validation->set_rules('username', $this->lang->line('username'), 'callback_username_check');
      $this->form_validation->set_rules('password', $this->lang->line('password'), 'callback_password_check');
      if ($this->form_validation->run() == FALSE)
      {
        $this->load->view('admin/login');
      }
      else
      {
        $this->session->set_userdata('logged_in', TRUE);
        redirect('admin/main');
      }
    } 
  }
  
  
  /**
  * Callback function for username check
  *
  * @access   public
  * @param    string
  * @return   boolean
  */
  function username_check($str)
	{
		if ($str != $this->config->item('admin_username'))
		{
			$this->form_validation->set_message('username_check', $this->lang->line('username_is_incorrect'));
			return FALSE;
		}
		else
		{
			return TRUE;
		}
	}
  
  
  /**
  * Callback function for password check
  *
  * @access   public
  * @param    string
  * @return   boolean
  */
  function password_check($str)
  {
    if ($str == $this->config->item('admin_password'))
    {
      return TRUE;
    }
    else
    {
      $this->form_validation->set_message('password_check', $this->lang->line('password_is_incorrect'));
      return FALSE;
    }
  }
  
  
  /**
  * Function: Logout
  *
  * @access	  public
  * @param    string
  * @return   boolean
  */
  function logout()
  {
    $this->session->unset_userdata('logged_in');
    redirect('admin/login');
  }
  
}
?>