<?php
/** 
* Pages administration
*
* Controller for displaying, modifying and deleting pages.
* 
* @package      ezpx 
* @subpackage   Admin
* @author       Peter Zimon 
*/ 
class Pages extends Controller {
  
  
  var $header_data;
  
  
  function Pages()
  {
    parent::Controller();
    
    $this->load->database();
    $this->load->model('Ezpx');
    
    $libraries = array('session', 'form_validation');
    $this->header_data['header_title'] = $this->lang->line('pages');
    $this->load->library($libraries);
    
    $helpers = array('form');
    $this->load->helper($helpers);
    
    // Load FCKeditor
    $this->load->library('fckeditor', array('instanceName' => 'body'));
    
    if ($this->session->userdata('logged_in') != TRUE) 
    {
      redirect('admin/login');
    }
  }
  
  
  /** 
  * Display existing pages
  *
  * @access public
  */ 
  function index()
  {
    $pages = $this->Ezpx->get_all_pages();
    $data['pages'] = $pages->result_array();
    
    // Get subpages
    if (is_array($data['pages']))
    {
      foreach ($data['pages'] as $key => $value) {
        $subpages = $this->Ezpx->get_all_subpages($value['id']);
        $data['pages'][$key]['subpages'] = $subpages->result_array();
      }
    }
    
    // Call views
    $this->load->view('admin/header', $this->header_data);
    $this->load->view('admin/pages', $data);
    $this->load->view('admin/footer');
  }
  
  
  /** 
  * Maintain page
  *
  * Insert new page and modify one. Display mainenance form, validate post data
  * and call necessary database methods.
  * 
  * @access public
  */ 
  function maintain()
  {
    $id = $this->uri->segment(4);
    
    // Form validation
    $this->form_validation->set_rules('title', $this->lang->line('title'), 'required');
    
    if ($this->form_validation->run() == TRUE && ! $this->config->item('demo_mode'))
    {
      if ($id == 'new')
      {
        if ($this->Ezpx->insert_page($this->input))
        {
          redirect('admin/pages');
        } 
        else
        {
          $data['error'] = 'An error occured during database insertion.';
        }
      }
      else
      {
        if ($this->Ezpx->update_page($id, $this->input))
        {
          redirect('admin/pages');
        } 
        else
        {
          $data['error'] = 'An error occured during database update.';
        }
      }
      
      // Display error
      $this->load->view('admin/header', $this->header_data);
      $this->load->view('admin/error', $data);
      $this->load->view('admin/footer');
      
    }
    else
    {
    
      $data['action'] = 'admin/pages/maintain/' . $id;
      
      $existing_pages = $this->Ezpx->get_all_pages();
      $existing_pages = $existing_pages->result_array();
      $data['existing_pages'][0] = $this->lang->line('the_page_has_no_parent');
      if (is_array($existing_pages))
      {
        foreach ($existing_pages as $trash => $value) {
          if ($value['id'] != $id)
          {
            $data['existing_pages'][$value['id']] = $value['title'];
          }
        }
      }
      $data['options_open'] = FALSE;
      
      if ($id == 'new') 
      {
        $data['title'] = '';
        $data['body'] = $this->input->post('body');
        $data['parent_id'] = $this->input->post('parent_id');
        if ($this->input->post('go') && $data['parent_id'] != 0)
        {
          $data['options_open'] = TRUE;
        }
      }
      else
      {
        $page = $this->Ezpx->get_page($id);
        $page = $page->row_array();
        $data['id'] = $id;
        $data['title'] = $page['title'];
        
        if ($this->input->post('go'))
        {
          $data['body'] = $this->input->post('body');
          $data['parent_id'] = $this->input->post('parent_id');
        }
        else
        {
          $data['body'] = $page['body'];
          $data['parent_id'] = $page['parent_id'];
        }
        
        if ($data['parent_id'] != 0)
        {
          $data['options_open'] = TRUE;
        }
      }
      
      // Call maintenance view
      $this->load->view('admin/header', $this->header_data);
      $this->load->view('admin/page_maintain', $data);
      $this->load->view('admin/footer');
    }
  }
  
  
  /**
  * Delete page
  *
  * @access  public
  */
  function delete()
  {
    // Check demo mode
    if ($this->config->item('demo_mode'))
    {
      redirect('admin/pages');
    }
  
    $id = $this->uri->segment(4);
    $error['error'] = '';
    
    if ( ! $this->Ezpx->delete_page($id))
    {
      $error['error'] = 'An error occured during deleting page.';
    }
    
    // If error occured -> display it
    if ($error['error'])
    {
      $this->load->view('admin/header', $this->header_data);
      $this->load->view('admin/error', $error);
      $this->load->view('admin/footer');
    }
    else
    {
      redirect('admin/pages');
    }    
  }
}
?>