<?php
/** 
* Photo administration
*
* Controller for displaying, modifying and deleting photos.
* 
* @package      ezpx 
* @subpackage   Admin
* @author       Peter Zimon 
*/ 
class Photos extends Controller {
  
  var $header_data;
  var $photos_per_page;
  var $comments_per_page;
  var $settings;
  
  function Photos() 
  {
    parent::Controller();
    
    $this->load->database();
    $this->header_data['header_title'] = $this->lang->line('photos');
    $this->photos_per_page = 12;
    $this->comments_per_page = 10;
    $this->load->model('Ezpx');
    
    $libraries = array('session', 
                       'form_validation', 
                       'pagination', 
                       'image_lib');
    $this->load->library($libraries);
    
    // Load FCKeditor
    $this->load->library('fckeditor', array('instanceName' => 'body'));
    
    $helpers = array('form', 
                     'ezpx');
    $this->load->helper($helpers);
    
    $stgs = $this->Ezpx->get_settings();
    $this->settings = $stgs ->row_array();
    
    if ($this->session->userdata('logged_in') != TRUE) 
    {
      redirect('admin/login');
    }
    
    // Set default view for photos
    if ( ! $this->settings['photo_view'])
    {
      $this->settings['photo_view'] = 'LIST';
    }
  }

  
  /**
  * Default method
  *
  * Display photo thumbnails with pagination
  * 
  * @access public
  */
  function index() 
	{	 
	  // Check if we're in normal mode
    switch ($this->settings['mode'])
    {
      case 'FLICKR':
        $this->load->view('admin/header', $this->header_data);
        $this->load->view('admin/go_to_flickr');
        $this->load->view('admin/footer');
        return;
        break;
    }
    
    // Config pagination
    unset($config);
    $config['base_url'] = site_url('admin/photos/index');
    $config['total_rows'] = $this->Ezpx->count_all($this->config->item('db_table_prefix') . 'photos');;
    $config['per_page'] = $this->photos_per_page;
    $config['uri_segment'] = 4;
    $this->pagination->initialize($config);
    
    // Gather photolist
    $start = $this->uri->segment(4, 0);
    $photos = $this->Ezpx->get_all_photos($this->photos_per_page, $start);
    $data['photos'] = $photos->result_array();
    
    foreach($data['photos'] as $key => $photo) {
      if (file_exists('./content/photos/tn_' . $photo['photo_file']))
      {
        $data['photos'][$key]['photo_file'] = 'tn_' . $photo['photo_file'];
      }
      elseif (file_exists('./content/photos/resized_' . $photo['photo_file']))
      {
        $data['photos'][$key]['photo_file'] = 'resized_' . $photo['photo_file'];
      }
      else
      {
        $data['photos'][$key]['photo_file'] = $photo['photo_file'];
      }
    }
    
    // Calculate thumbnail width and description margin
    $data['thumbnail_width'] = '';
    if ($this->settings['thumbnail_width'] > 100)
    {
      $data['thumbnail_width'] = ' width="100"';
      $data['description_margin'] = '130px';
    }
    else
    {
      $data['thumbnail_width'] = ' width="' . $this->settings['thumbnail_width'] . '"';
      $data['description_margin'] = ($this->settings['thumbnail_width'] + 30) . 'px';
    }
    
    $data['photo_view'] = $this->settings['photo_view'];
    
    // Call views
    $this->load->view('admin/header', $this->header_data);
    $this->load->view('admin/photos', $data);
    $this->load->view('admin/footer');
	}
	
	
	/**
	* Preapre tags
	* 
	* @access private
	* @return string
	*/
	function _prepare_tags()
	{
	  $tags = $this->Ezpx->get_tags_count();
	  $tags = $tags->result_array();
	  $all = count($tags);
	  $tags_string = '';
	  if ($all)
	  {
	    $i = 0;
      foreach ($tags as $tag)
      {
        $tags_string .= '"' . $tag['tag'] . '"';
        $i++;
        if ($i < $all) 
        {
          $tags_string .= ", ";
        }
      }
	  }
    return $tags_string;
	}
	
	
	/**
  * Upload photo and modify existing ones
  *
  * By default the upload form is displayed. If an error occures during
  * the form validation the error messages are displayed. 
  * 
  * Checks if all required fields are filled and uploads the file.
  * If the file is uploaded without errors it calls the model's
  * corresponding method to insert the new file info to the database.
  *
  * @access public
  */
  function upload()
  {
    // Check if we're in normal mode
    switch ($this->settings['mode'])
    {
      case 'FLICKR':
        $this->load->view('admin/header', $this->header_data);
        $this->load->view('admin/go_to_flickr');
        $this->load->view('admin/footer');
        return;
        break;
    }
    
    $this->form_validation->set_rules('title', $this->lang->line('photo_title'), 'required');
    $this->form_validation->set_rules('tags', '', '');
    $this->form_validation->set_rules('post_date', '', '');
    $this->form_validation->set_rules('gallery_link', '', '');
    
    $error['error'] = '';
        
    // Form validation
    if ($this->form_validation->run() == TRUE && ! $this->config->item('demo_mode'))
    {
      // Upload file
      unset($config);
      $config['upload_path'] = './content/photos/';
  		$config['allowed_types'] = 'gif|jpg|png';
  		$config['max_size']	= '10240';
  		$config['max_width']  = '5000';
  		$config['max_height']  = '5000';
  		$config['encrypt_name'] = TRUE;
  		
  		$this->load->library('upload', $config);
  	
  		if ( ! $this->upload->do_upload('photo_file'))
  		{
  			$error['error'] .= $this->upload->display_errors();
  		}	
  		
  		
  		// Set upload data
  		if ( ! $error['error'])
  		{
  		  $upload_data = $this->upload->data();
  		  if ( ! $upload_data['file_name'])
  		  {
  		    $error['error'] .= 'Cannot retrieve uploaded file name.';
  		  }
  		}
  		
  		
  		// Insert photo into database
  		if ( ! $error['error'])
  		{
  		  $tag_string = $this->input->post('tags');
  		  if (is_array($tags = explode(',', $tag_string)))
  		  {
  		    foreach($tags as $key => $tag)
  		    {
  		      $tags[$key] = trim($tag);
  		    }
  		  }
  		  $tags = array_unique($tags);
		    if ( ! $this->Ezpx->insert_photo($upload_data['file_name'], $tags, $this->input))
		    {
		      $error['error'] .= 'Error during databse insert.';
		    }
  		}
  		
  		
  		// Create resized image
  		if ( ! $this->settings['dynamic_resize']) 
  		{
  		  if ( ! $error['error'])
  	    {
  	      if ($this->settings['resize_to'] < $upload_data['image_width']) 
  	      {	        
  	        if ( ! ep_resize_image($this->config, $upload_data, $this->settings['resize_to'], $this->image_lib))
  	        {
  	          $error['error'] .= 'Error when resizing image file!<br />';
  	        }
  	      }
  	    }
	    }
	    
	    
	    // Create thumbnail image
	    if ( ! $error['error'])
	    {        
        if ( ! ep_create_thumbnail($this->config, $upload_data, $this->settings, $this->image_lib))
        {
          $error['error'] .= 'Error when creating thumbnail!<br />';
        }
        
	    }
	    
  		
  		// Read and save EXIF info **** TO BE IMPLEMENTED ****
      if ( ! $error['error'])
      {
        if (function_exists('exif_read_data') && $upload_data['image_type'] == 'jpeg')
        {
          if ($exif = @exif_read_data('./content/photos/' . $upload_data['file_name']))
          {
            /* TO BE IMPLEMENTED */
          }
        }
      }
      
      
      // Redirect to success page if no error occured
      if ( ! $error['error'])
      {
        redirect('admin/photos/upload_success');
      }
    }
    
    // Display error page if error occured
    $this->load->view('admin/header', $this->header_data);
    $data['error'] = $error['error'];
    $data['title'] = '';
    $data['body'] = $this->input->post('body');
    $data['action'] = 'admin/photos/upload';
    $data['file'] = '';
    $data['tags'] = '';
    $data['post_year'] = '';
    $data['post_month'] = '';
    $data['post_day'] = '';
    $data['post_year_options'] = '';
    $data['post_month_month'] = '';
    $data['post_day_options'] = '';
    $data['upload_date'] = $this->lang->line('today');
    $data['gallery_link'] = 'http://';
    $data['page_title'] = $this->lang->line('upload_photo');
    $data['tags_ar'] = $this->_prepare_tags();    
    $this->load->view('admin/photo_upload', $data);
    $this->load->view('admin/footer');
    
  }
  
  
  /**
  * Upload success
  * 
  * If no error occured while uploading, then a success screen is 
  * displayed.
  *
  * @access public
  */
  function upload_success()
  {
    $this->load->view('admin/header', $this->header_data);
    $this->load->view('admin/photo_upload_success');
    $this->load->view('admin/footer');
  }
  
  
  /**
  * Delete photo
  *
  * Deletes a photo from the disk and the database.
  *
  * @access public
  */
  function delete()
  {
    // Check demo mode
    if ($this->config->item('demo_mode'))
    {
      redirect('admin/photos');
    }
  
    $id = $this->uri->segment(4, 0);
    $error['error'] = '';
    
    // Gather file info
    $photo = $this->Ezpx->get_photo($id);
    $photo = $photo->result_array();
    $photo = $photo[0];
    $file_name = $photo['photo_file'];
    
    
    // Delete file
    if ( ! unlink('./content/photos/' . $file_name))
    {
      $error['error'] .= 'An error occured while deleting file. Please try again.';
    }
    
    
    // Delete resized and thumbnail image
    if ( ! $error['error']) 
    {
      // Delete resized if exists
      if (file_exists('./content/photos/resized_' . $file_name))
      {
        @unlink('./content/photos/resized_' . $file_name);
      }
      
      // Delete thumbnail if exists
      if (file_exists('./content/photos/tn_' . $file_name))
      {
        @unlink('./content/photos/tn_' . $file_name);
      }
    }
    
    // Delete tags
    if ( ! $error['error'])
    {
      if ( ! $this->Ezpx->delete_tags($id) )
      {
        $error['error'] = 'Error during delete tags from database';
      }
    }
    
    
    // Delete comments for photo
    if ( ! $error['error'])
    {
      if ( ! $this->Ezpx->delete_comments_for_photo($id) )
      {
        $error['error'] = 'Error during delete comments from database';
      }
    }
    
    
    // Delete database entry
    if ( ! $error['error'])
    {
      if ( ! $this->Ezpx->delete_photo($id))
      {
        $error['error'] = 'An error occured while deleting photo. Please try again.';
      }
    }
    
    // If error occured -> display it
    if ($error['error'])
    {
      $this->load->view('admin/header', $this->header_data);
      $this->load->view('admin/error', $error);
      $this->load->view('admin/footer');
    }
    else
    {
      redirect('admin/photos');
    }
  }
  
  
  /**
  * Modify photo
  *
  * Updates a photo in the database.
  *
  * @access public
  */
  function modify()
  {
    $id = $this->uri->segment(4, 0);
    $go = $this->uri->segment(5);
    $photo = $this->Ezpx->get_photo($id);
    $photo = $photo->result_array();
    $photo = $photo[0];
    
    // Validate form
    $this->form_validation->set_rules('title', $this->lang->line('photo_title'), 'required');
    $this->form_validation->set_rules('tags', '', '');
    $this->form_validation->set_rules('post_date', '', '');
    $this->form_validation->set_rules('gallery_link', '', '');
    
    if ($this->form_validation->run() == TRUE && ! $this->config->item('demo_mode'))
    {
      $tag_string = $this->input->post('tags');
      $tags = explode(',', $tag_string);
      if (is_array($tags = explode(',', $tag_string)))
		  {
		    foreach($tags as $key => $tag)
		    {
		      $tags[$key] = trim($tag);
		    }
		  }
		  $tags = array_unique($tags);
      if ($this->Ezpx->update_photo($id, $tags, $this->input))
      {
        redirect('admin/photos');
      }
      else
      {
        $data['error'] = 'An error occured when updating database entry.';
        $this->load->view('admin/header', $this->header_data);
        $this->load->view('admin/error', $error);
        $this->load->view('admin/footer');
      }
    } 
    else 
    {
      // Photo
      $data = $photo;
      if ($this->input->post('go'))
      {
        $data['body'] = $this->input->post('body');
      }
      if (!isset($data['error'])) $data['error'] = '';
      $data['action'] = 'admin/photos/modify/' . $id . '/go';
      if (file_exists('./content/photos/tn_' . $photo['photo_file']))
      {
        $data['file'] = base_url() . '/content/photos/tn_' . $photo['photo_file'];
      }
      elseif (file_exists('./content/photos/resized_' . $photo['photo_file']))
      {
        $data['file'] = base_url() . '/content/photos/resized_' . $photo['photo_file'];
      }
      else
      {
        $data['file'] = base_url() . '/content/photos/' . $photo['photo_file'];
      }
      
      // Calculate thumbnail width
      $data['thumbnail_width'] = '';
      if ($this->settings['thumbnail_width'] > 190)
      {
        $data['thumbnail_width'] = ' width="190"';
      }
      
      // Upload date
      $upload_date = ep_parse_date($photo['upload_date']);
      $data['upload_date'] = $upload_date['year'] . '-' . $upload_date['month'] . '-' . $upload_date['day'];
      
      // Tags
      $tags = $this->Ezpx->get_tags($id);
      $all_tags = $tags->num_rows(); 
      $i = 0;
      $tags = $tags->result_array();
      $data['tags'] = '';
      foreach($tags as $tag)
      {
        $data['tags'] .= $tag['tag'];
        $i++;
        if ($i < $all_tags)
        {
          $data['tags'] .= ', ';
        }
      }
        
      // Comments
      $start = (int)$this->uri->segment(5, 0);
      
      $config['base_url'] = site_url('admin/photos/modify/' . $id);
      $config['total_rows'] = $this->Ezpx->count_all_comments($id);
      $config['per_page'] = $this->comments_per_page;
      $config['uri_segment'] = 5;
      
      $this->pagination->initialize($config);
      
      $data['comment_pager'] = $this->pagination->create_links();
      
      $comments = $this->Ezpx->get_all_comments($id, $this->comments_per_page, $start);
      $data['comments'] = $comments->result_array();
      
      $data['page_title'] = $this->lang->line('modify_photo');
      
      // Tags for autocomplete
      $data['tags_ar'] = $this->_prepare_tags();
      
      // Call views
      $this->load->view('admin/header', $this->header_data);
      $this->load->view('admin/photo_upload', $data);
      $this->load->view('admin/footer');
    }
  }
  
  
  /** 
  * Rotate photo
  *
  * Rotates a photo CC or CCW.
  * 
  * @access public
  */ 
  function rotate()
  {
    $id = $this->uri->segment(4);
    $wise = $this->uri->segment(5);
    $error = '';
    
    // Check demo mode
    if ($this->config->item('demo_mode'))
    {
      redirect('admin/photos/modify/' . $id);
    }
    
    $photo = $this->Ezpx->get_photo($id);
    $photo = $photo->result_array();
    $photo = $photo[0];
    
    if (!$id OR !$wise)
    {
      redirect('admin/main');
    }
    
    // Process original
    unset($config);
    $this->image_lib->clear();
    $config['image_library'] = $this->config->item('image_library');
    $config['library_path'] = $this->config->item('library_path');
    $config['source_image'] = './content/photos/' . $photo['photo_file'];
    $config['rotation_angle'] = ($wise == 'ccw') ? '90' : '270';
    
    $this->image_lib->initialize($config);
    
    if ( ! $this->image_lib->rotate())
    {
      $error['error'] = $this->image_lib->display_errors();
    }
    
    // Process resized
    if ( ! $this->settings['dynamic_resize']) 
    {
      if (file_exists('./content/photos/resized_' . $photo['photo_file']))
      {
        $this->image_lib->clear();
        $config['source_image'] = './content/photos/resized_' . $photo['photo_file'];
      
        $this->image_lib->initialize($config);
      
        if ( ! $this->image_lib->rotate())
        {
          $error['error'] = $this->image_lib->display_errors();
        }
      }
    }
    
    // Process thumbnail
    if (file_exists('./content/photos/tn_' . $photo['photo_file']))
    {
      $this->image_lib->clear();
      $config['source_image'] = './content/photos/tn_' . $photo['photo_file'];
      
      $this->image_lib->initialize($config);
      
      if ( ! $this->image_lib->rotate())
      {
        $error['error'] = $this->image_lib->display_errors();
      }
    }
    
    // Redirect if no errors occured
    if (!$error)
    {
      redirect('admin/photos/modify/' . $id);
    }
    
    // Display errors
    $this->load->view('admin/header', $this->header_data);
    $this->load->view('admin/error', $error);
    $this->load->view('admin/footer');
  }
  

  /** 
  * Delete comment
  *
  * Delete comment from database
  * 
  * @access public
  */ 
  function delete_comment() 
  {
    $comment_id = $this->uri->segment(4, 0);
    $photo_id = $this->uri->segment(6);
    if ($this->Ezpx->delete_comment($comment_id))
    {
      redirect(site_url('admin/photos/modify/' . $photo_id));
    }
    else
    {
      $error['error'] = 'Error when deleting comment from database.';
      // Display errors
      $this->load->view('admin/header', $this->header_data);
      $this->load->view('admin/error', $error);
      $this->load->view('admin/footer');
    }
  }
  
  
  /**
  * Set view of photo listing
  * 
  * @access public
  */
  function set_view()
  {
    $view = $this->uri->segment(4, 'list');
    switch ($view)
    {
      case 'grid':
        $this->Ezpx->update_photo_view('GRID');
        break;
      default:
        $this->Ezpx->update_photo_view('LIST');
        break;
    }
    redirect('admin/photos');
  }
    
}
?>