<?php
/** 
* Settings
*
* Handle global settings that are saved in database and can be modified.
* 
* @package      ezpx 
* @subpackage   Admin
* @author       Peter Zimon 
*/ 
class Settings extends Controller {
  
  var $header_data;
  
  
  function Settings()
  {
    parent::Controller();
    
    $this->load->database();
    $this->load->model('Ezpx');
    
    $libraries = array('session', 'form_validation', 'image_lib');
    $this->header_data['header_title'] = $this->lang->line('settings');
    $this->load->library($libraries);
    
    $helpers = array('form', 'ezpx');
    $this->load->helper($helpers);
    
    if ($this->session->userdata('logged_in') != TRUE) 
    {
      redirect('admin/login');
    }
  }
  
  
  /** 
  * Default function
  *
  * Display settings form with pre-filled values from database. If no values are
  * in the database insert one default row.
  * 
  * @access public
  */ 
  function index()
  {
    // Get settings from database
    $settings = $this->Ezpx->get_settings();
    $data = $settings->row_array();
    
    $data['comments_allowed'] = (boolean)$data['comments_allowed'];
    $data['rss'] = (boolean)$data['rss'];
    $data['dynamic_resize'] = (boolean)$data['dynamic_resize'];
    $data['display_menu_photos'] = (boolean)$data['display_menu_photos'];
    $data['display_menu_archives'] = (boolean)$data['display_menu_archives'];
    $data['display_menu_links'] = (boolean)$data['display_menu_links'];
    $data['display_menu_pages'] = (boolean)$data['display_menu_pages'];
    
    $error = '';
    
    // Form validation
    $this->form_validation->set_rules('photos_per_page', $this->lang->line('photos_per_page'), 'required|is_natural_no_zero');
    $this->form_validation->set_rules('archive_per_page', $this->lang->line('archive_thumbnails_per_page'), 'required|is_natural_no_zero');
    $this->form_validation->set_rules('dynamic_resize', '', '');
    $this->form_validation->set_rules('resize_to', $this->lang->line('resize_photos_to'), 'is_natural_no_zero');
    $this->form_validation->set_rules('thumbnail_width', $this->lang->line('thumbnail_width'), 'required|is_natural_no_zero');
    $this->form_validation->set_rules('thumbnail_height', $this->lang->line('thumbnail_height'), 'required|is_natural_no_zero');
    $this->form_validation->set_rules('blog_title', $this->lang->line('blog_title'), 'required');
    $this->form_validation->set_rules('comments_allowed', '', '');
    $this->form_validation->set_rules('display_menu_photos', '', '');
    $this->form_validation->set_rules('display_menu_archives', '', '');
    $this->form_validation->set_rules('display_menu_links', '', '');
    $this->form_validation->set_rules('display_menu_pages', '', '');
    $this->form_validation->set_rules('mode', $this->lang->line('photoblog_mode'), 'required');
    $this->form_validation->set_rules('rss', '', '');
    
    if ($this->form_validation->run() == TRUE && ! $this->config->item('demo_mode')) 
    {
      // Call batch resizer
      if ( ! $this->input->post('dynamic_resize')) 
      {
        if ($this->input->post('resize_to') != $data['resize_to'] OR
            $this->input->post('dynamic_resize') != $data['dynamic_resize'])
        {
          $resize_result = $this->_batch_resize($this->input->post('resize_to'));
          if ($resize_result !== TRUE)
          {
            $error['error'] = 'Error during batch resize.<br />' . $resize_result;
          }
        }
      }
      
      // Dynamic resize is turned on -> all resized photos can be deleted
      else
      {
        $this->_delete_resized();
      }
      
      // Call batch thumbnailer
      if ($this->input->post('thumbnail_width') != $data['thumbnail_width'] OR 
          $this->input->post('thumbnail_height') != $data['thumbnail_height'])
      {
        $thumbnail_result = $this->_batch_thumbnail($this->input->post('thumbnail_width'), $this->input->post('thumbnail_height'));
        if ($thumbnail_result !== TRUE)
        {
          $error['error'] = 'Error during batch thumbnailer.<br />' . $thumbnail_result;
        }
      }
    
      // Update database
      if ( ! $error)
      {
        if ( ! $this->Ezpx->update_settings($this->input)) 
        {
          $error['error'] = 'Error during updating database.<br />';
        }
      }
      
      
      // Display success or error message
      if ( ! $error) 
      {
        $this->load->view('admin/header', $this->header_data);
        $this->load->view('admin/settings_success');
        $this->load->view('admin/footer');
      }
      else
      {
        $data['error'] = $error['error'];
        $this->load->view('admin/header', $this->header_data);
        $this->load->view('admin/error', $data);
        $this->load->view('admin/footer');
      }
    }
    else
    {
      
      // Set photo options open if error occures
      if (form_error('photos_per_page') OR
          form_error('archives_per_page') OR
          form_error('resize_to') OR
          form_error('thumbnail_width') OR
          form_error('thumbnail_height'))
      {
        $data['photo_options_open'] = TRUE;
      }
      else
      {
        $data['photo_options_open'] = FALSE;
      }
      
      // Set blogging options open if error occures
      if (form_error('comments_allowed') OR
          form_error('blog_title'))
      {
        $data['blogging_options_open'] = TRUE;
      }
      else
      {
        $data['blogging_options_open'] = FALSE;
      }
      
      // Set layout options open if error occures
      if (TRUE)
      {
        $data['layout_options_open'] = FALSE;
      }
      
      // Set mode options open if error occures
      if ($this->uri->segment(4) == 'open_mode')
      {
        $data['mode_options_open'] = TRUE;
      }
      else
      {
        $data['mode_options_open'] = FALSE;
      }
      
      // Auto open
      switch ($this->uri->segment(4))
      {
        case 'open_all':
          $data['photo_options_open'] = TRUE;
          $data['blogging_options_open'] = TRUE;
          break;
        case 'open_first':
          $data['blogging_options_open'] = TRUE;
          break;
      }
      
      // Gather flickr related data
      $flickr_data = $this->Ezpx->get_flickr_settings();
      $data = array_merge($data, $flickr_data);
      
      $this->load->view('admin/header', $this->header_data);
      $this->load->view('admin/settings', $data);
      $this->load->view('admin/footer');
    }
  }
  
  
  /** 
  * Batch resize
  *
  * If any of the resize to or thumbnail sizes are changed all resized images 
  * and thumbnails have to be updated
  * 
  * @access private
  * @param  string
  * @return boolean/array
  */
  function _batch_resize($resize_to)
  {
    $error['error'] = '';
    
    $photos = $this->Ezpx->get_all_photos();
    $photos = $photos->result_array();
    
    // If resize size is blank, delete all existing resized image
    if ($resize_to == '')
    {
      foreach($photos as $photo)
      {
        $resized_file = './content/photos/resized_' . $photo['photo_file'];
        if (file_exists($resized_file))
        {
          @unlink($resized_file);
        }
      }
      return TRUE;
    }
    
    // If resize value is maintained, delete all existing resized image create new ones
    else
    {
      foreach($photos as $photo)
      {
        if ( ! file_exists('./content/photos/' . $photo['photo_file']))
        {
          continue;
        }
        
        $resized_file = './content/photos/resized_' . $photo['photo_file'];
        if (file_exists($resized_file))
        {
          @unlink($resized_file);
        }
        
        $image_props = getimagesize('./content/photos/' . $photo['photo_file']);
        
        if (($image_props[0] >= $image_props[1] && $resize_to < $image_props[0]) OR
            ($image_props[0] <= $image_props[1] && $resize_to < $image_props[1])) 
        {          
          $file_data['file_name'] = $photo['photo_file'];
          
          if ( ! ep_resize_image($this->config, $file_data, $resize_to, $this->image_lib))
	        {
	          break;
	          $error['error'] .= 'Error when resizing image file!<br />';
	        }
        }
      }
    
      if ( ! $error['error'])
      {
        return TRUE;
      }
      else
      {
        return $error;
      }
    
    }
  }
  
  
  /** 
  * Delete resized
  *
  * When dynamic resize is turned on, all previously uploaded resized
  * photos can be deleted.
  * 
  * @access private
  * @param  string
  * @return boolean/array
  */
  function _delete_resized()
  {
    $error['error'] = '';
    
    $photos = $this->Ezpx->get_all_photos();
    $photos = $photos->result_array();
    
    foreach($photos as $photo)
    {
      $resized_file = './content/photos/resized_' . $photo['photo_file'];
      if (file_exists($resized_file))
      {
        @unlink($resized_file);
      }
    }
    return TRUE;
    
  }
  
  
  /** 
  * Batch thumbnailer
  *
  * If any of the resize to or thumbnail sizes are changed all resized images 
  * and thumbnails have to be updated
  * 
  * @access private
  * @param  string
  * @return boolean/array
  */
  function _batch_thumbnail($thumbnail_width, $thumbnail_height)
  {
    $error['error'] = '';
    
    $photos = $this->Ezpx->get_all_photos();
    $photos = $photos->result_array();
  
    foreach($photos as $photo)
    {
      if ( ! file_exists('./content/photos/' . $photo['photo_file']))
      {
        continue;
      }
      
      $thumbnail_file = './content/photos/tn_' . $photo['photo_file'];
      if (file_exists($thumbnail_file))
      {
        @unlink($thumbnail_file);
      }
      
      $image_props = getimagesize('./content/photos/' . $photo['photo_file']);
      
      $file_data['file_name']           = $photo['photo_file'];
      $file_data['image_width']         = $image_props[0];
      $file_data['image_height']        = $image_props[1];
      $tn_settings['thumbnail_width']   = $thumbnail_width;
      $tn_settings['thumbnail_height']  = $thumbnail_height;
      
      if ( ! ep_create_thumbnail($this->config, $file_data, $tn_settings, $this->image_lib))
      {
        $error['error'] .= 'Error when creating thumbnail!<br />';
        break;
      } 
    }
  
    if ( ! $error['error'])
    {
      return TRUE;
    }
    else
    {
      return $error;
    }
  
  }
  
}
?>