<?php
/** 
* Archives
*
* Controller responsible for gathering archive thumbnail photos. Currently supported
* archive types: all thumbnails with pagination, monthly archives, tagcloud. Monthly
* archives and tagcloud leads to searchpage. 
* 
* @package      ezpx 
* @author       Peter Zimon 
*/ 
class Archives extends Controller {

  var $settings;

  function Archives() 
  {
    parent::Controller();
    
    $this->load->database();
    $this->load->model('Ezpx');
    
    $libraries = array('parser', 'pagination', 'wordcloud');
    $this->load->library($libraries);
    
    $stgs = $this->Ezpx->get_settings();
    $this->settings = $stgs->row_array();
    
    // Get FLICKR settings
    if ($this->settings['mode'] == 'FLICKR')
    {
      $this->flickr_settings = $this->Ezpx->get_flickr_settings();
      if ( ! $this->flickr_settings)
      {
        die('Your flickr settings are not complete. Please provide all necessary flickr information.');
      }
      $config['user_id'] = $this->flickr_settings['flickr_user_id'];
      $this->load->library('flickr', $config);
    }    
  }
  
  
  /**
  * Default function
  *
  * Display archive categories 
  *
  * @access   public
  */
  function index()
  {
    // Get global tags <- this must be called first for data displaying
    $data = $this->Ezpx->get_global_tags();
    
    // Get global ezpx configuration
    $data['cfg_wrapper']   = $this->config->item('wrapper');
    $data['tpl_base_dir']  = 'content/templates/' . $this->config->item('template');
    $data['tpl_base_url']  = base_url() . 'content/templates/' . $this->config->item('template');
    
    // Get data depending on photoblog mode
    switch ($this->settings['mode'])
    {
      case 'FLICKR':
        // Convert start 2 flickr page numba
        $start = ($this->uri->segment(3, 0) / $this->settings['archive_per_page']) + 1;
        $flickr_archives = $this->_get_archive_thumbnails_flickr($start);
        $total = isset($flickr_archives['total']) ? $flickr_archives['total'] : 0;
                
        // Get archive thumbnails
        $data['archive_thumbnails'] = isset($flickr_archives['archive_thumbnails']) ? $flickr_archives['archive_thumbnails'] : '';
        
        // Get archives by month for flickr -> NOT SUPPORTED
        $data['archive_monthly'] = '';
        
        // Get tagcloud
        $data['archive_tagcloud'] = $this->_get_archives_tagcloud_flickr();
        break;
      
      // Normal behaviour
      default:
        // Config pagination
        $total = $this->Ezpx->count_all($this->config->item('db_table_prefix') . 'photos');
        $start = $this->uri->segment(3, 0);
        
        // Get archive thumbnail photos
        $data['archive_thumbnails'] = $this->_get_archive_thumbnails_normal($start);

        // Get archives by month
        $data['archive_monthly'] = $this->_get_archives_monthly_normal();

        // Get tagcloud
        $data['archive_tagcloud'] = $this->_get_archives_tagcloud_normal();
        break;
    }
    
    $config['base_url'] = site_url('archives/index');
    $config['total_rows'] = $total;
    $config['per_page'] = $this->settings['archive_per_page'];
    $config['uri_segment'] = 3;
    $config['first_link'] = '&laquo;&laquo; First';
    $config['last_link'] = 'Last &raquo;&raquo;';
    $config['next_link'] = '<strong>Next &raquo;</strong>';
    $config['prev_link'] = '<strong>&laquo; Previous</strong>';
    $config['num_links'] = 1;
        
    $this->pagination->initialize($config);
    
    // Get pagination data
    $data['thumbnail_pager'] = $this->pagination->create_links();
    
    // Parse output
    $this->parser->parse('public/archives', $data);
  }
  
  
  /**
  * Retrieve archives when normal mode
  * 
  * @access private
  * @param number
  * @return array
  */
  function _get_archive_thumbnails_normal($start)
  {
    $photos = $this->Ezpx->get_all_photos($this->settings['archive_per_page'], $start);
    $photos = $photos->result_array();
    $archive_thumbnails = '';
    
    foreach ($photos as $photo)
    {
      if (file_exists('./content/photos/tn_' . $photo['photo_file']))
      {
        $archive_thumbnails .= anchor('photo/show/' . $photo['id'], '<img src="' . base_url() . '/content/photos/tn_' . $photo['photo_file'] . '" border="0">');
      }
    }
    
    return $archive_thumbnails;
  }
  
  
  /**
  * Retrieve monthly archives when normal mode
  * 
  * @access private
  * @return array
  */
  function _get_archives_monthly_normal()
  {
    $months = $this->Ezpx->get_archive_months();
    $archive_monthly = '';
    
    foreach ($months->result_array() as $month)
    {
      $display_year = substr($month['archive_month'], 0, 4);
      $display_month = substr($month['archive_month'], 5, 2);
      $archive_monthly .= ' ' . anchor('search/by/month/' . $display_year . '/' . $display_month, $display_year . '. ' . $display_month);
    }
    
    return $archive_monthly;
  }
  
  
  
  /**
  * Retrieve monthly archives when normal mode
  * 
  * @access private
  * @return array
  */
  function _get_archives_tagcloud_normal()
  {
    unset($config);
    $config['addlink'] = TRUE;
    $config['shuffle_words'] = TRUE;
    $config['base_url'] = site_url('search/by/tag');

    $this->wordcloud->initialize($config);

    // Adding words
    $tags = $this->Ezpx->get_tags_count();
    foreach ($tags->result_array() as $tag)
    {
      $this->wordcloud->addWord($tag['tag'], $tag['counter']);
    }

    return $this->wordcloud->showCloud();
  }
  
  
  /**
  * Retrieve archives when flickr mode
  * 
  * @access private
  * @param number
  * @return array
  */
  function _get_archive_thumbnails_flickr($page)
  {
    $flickr_thumbnails = $this->flickr->get_recent($page, $this->settings['archive_per_page']);
    
    if (is_array($flickr_thumbnails))
    {
      // Return all photo count
      $ret['total'] = $flickr_thumbnails['total'];
      
      // Build photos
      $ret['archive_thumbnails'] = '';
      foreach ($flickr_thumbnails['photos'] as $photo) {
        $ret['archive_thumbnails'] .= anchor('photo/show/' . $photo['id'], '<img src="' . $photo['photo_file'] . '" border="0">');
      }
      
      return $ret;
    }
  }
  
  /**
  * Retrieve tags when flickr mode
  * 
  * @access private
  * @param number
  * @return array
  */
  function _get_archives_tagcloud_flickr()
  {
    unset($config);
    $config['addlink'] = TRUE;
    $config['shuffle_words'] = TRUE;
    $config['base_url'] = site_url('search/by/tag');

    $this->wordcloud->initialize($config);

    // Adding words
    $tags = $this->flickr->get_tags();
    if (is_array($tags))
    {
      foreach ($tags['who']['tags']['tag'] as $tag)
      {
        $this->wordcloud->addWord($tag['_content'], $tag['count']);
      }

      return $this->wordcloud->showCloud();
    }
    return;
  }
}
?>