<?php
/** 
* Install
*
* Installer script for ezpx 
* 
* @package      ezpx 
* @author       Peter Zimon 
*/ 
class Install extends Controller {


  function Install() 
  {
    parent::Controller();
    
    $this->load->database();
    $this->load->dbutil();
    $this->load->model('Installer');
    
    // Check installation
    if ($this->uri->segment(2) != 'success' && $this->Installer->check_install())
    {
      redirect('main');
    }
    
    $libraries = array('pagination', 'form_validation');
    $this->load->library($libraries);
    
    $helpers = array('form', 'file');
    $this->load->helper($helpers);
  }
  
  
  /**
  * Default method
  * 
  * Decide whether clean install or upgrade is necessary and 
  * display the proper view
  *
  * @access public
  */
  function index()
  {    
    // Check if install is needed
    if ( ! $this->Installer->is_clean_install())
    {
      $view = 'public/upgrade';
      $data['db_procedure'] = $this->Installer->calculate_upgrade_diff();    
    }
    else
    {
      $view = 'public/install';
    }
    
    // Get settings
    $data['database'] = $this->db->database;
    $data['db_table_prefix'] = $this->config->item('db_table_prefix');
    
    $this->load->view($view, $data);
  }
  
  
  /**
  * Do install
  * 
  * Display the installer view
  *
  * @access public
  */
  function go()
  {    
    // Check if the button is pressed
    if ( ! $this->input->post('go_install') && ! $this->input->post('go_upgrade'))
    {
      redirect('install/error/push_button');
    }
    
    // Upgrade
    if ($this->input->post('go_upgrade'))
    {
      if ( ! $this->Installer->do_upgrade())
      {
        redirect('install/error/upgrade');
      }
      else
      {
        redirect('install/success/upgrade');
      }
    }    
    
    // Clean install
    if ($this->input->post('go_install'))
    {
      if ( ! $this->Installer->do_clean_install())
      {
        redirect('install/error/install');
      }
      else
      {
        redirect('install/success/install');
      }
    }
  }
  
  
  /**
  * Display error messages
  *
  * @access public
  */
  function error()
  {
    $error_type = $this->uri->segment(3);
    switch ($error_type)
    {
      case 'push_button':
        $data['msg'] = 'You have to push the big busty button to begin installation!';
        break;
        
      case 'query':
        $data['msg'] = 'An error occured during database table creation. Please run the installer again. '
                     . 'If this error still occures please try running the install sql script manually via '
                     . 'phpmyadmin. The script can be found in the _INSTALL_DEL_THIS_DIR/ezps.sql file.';
        break;
        
      case 'no_sql_file':
        $data['msg'] = 'No sql script file found in the install dir. This can easily be because you have '
                     . 'already run the install script and removed it from the server. If you would like to '
                     . 'run the install script (which will drop all your existing data) then copy the corresponding '
                     . 'directory (_INSTALL_DEL_THIS_DIR) to the server and run the installer again... BUT THIS IS '
                     . 'NOT RECOMMENDED!';
        break;
    }
    $this->load->view('public/install_error', $data);
  }
  
  
  /**
  * Display error messages
  *
  * @access public
  */
  function success()
  {
    $data['procedure'] = $this->uri->segment(3);
    $this->load->view('public/install_success', $data);
  }

  
}
?>