<?php
/** 
* Main
*
* Displays the latest photoblog entries according to ezpx configuration. 
* 
* @package      ezpx 
* @author       Peter Zimon 
*/ 
class Main extends Controller {


  var $settings;
  var $flickr_settings;
  

  function Main() 
  {
    parent::Controller();
    
    $this->load->database();
    $this->load->model('Ezpx');
    $this->load->library('session');
    
    // Check installation
    if ( ! $this->session->userdata('first_install_check'))
    {
      $this->load->model('Installer');
      if ( ! $this->Installer->check_install())
      {
        redirect('install');
      }
      else
      {
        $this->session->set_userdata('first_install_check', TRUE);
      }
    }
    
    $libraries = array('parser', 'pagination');
    $this->load->library($libraries);
    
    $stgs = $this->Ezpx->get_settings();
    $this->settings = $stgs->row_array();
    
    // Get FLICKR settings
    if ($this->settings['mode'] == 'FLICKR')
    {
      $this->flickr_settings = $this->Ezpx->get_flickr_settings();
      if ( ! $this->flickr_settings)
      {
        die('Your flickr settings are not complete. Please provide all necessary flickr information.');
      }
      $config['user_id'] = $this->flickr_settings['flickr_user_id'];
      $config['per_page'] = $this->settings['photos_per_page'];
      $this->load->library('flickr', $config);
    }
  }
  
  
  /**
  * Default method
  * 
  * Display latest photoblog entries with pagination on the bottom. 
  *
  * @access public
  */
  function index()
  {
    // Get global tags <- this must be called first for data displaying
    $data = $this->Ezpx->get_global_tags();
    
    // Get configuration
    $data['cfg_wrapper']   = $this->config->item('wrapper');
    $data['tpl_base_dir']  = 'content/templates/' . $this->config->item('template');
    $data['tpl_base_url']  = base_url() . 'content/templates/' . $this->config->item('template');
    
    // Init some variables
    $photos = $photos_result = $data['photos'] = array();
    $total = $start = 0;
    
    // Get photos depending on mode
    switch ($this->settings['mode'])
    {
      // FLICKR MODE
      case 'FLICKR':
        
        // $start = ($this->uri->segment(3, 0) / $this->settings['photos_per_page']);
        // if ($start * $this->settings['photos_per_page'] > $total OR $start == 0)
        // {
        //   $start = 1;
        // }
        $start = ($this->uri->segment(3, 0) / $this->settings['photos_per_page']) + 1;
        $flickr = $this->_get_flickr_photos($start);
        
        if (isset($flickr['photos']))
        {
          // Config pagination
          $total = $flickr['photos']['total'];
          
          // Get photos
          $photos = $flickr['photos']['photo'];
          $data['photos'] = $this->_standardize_photos_flickr($photos);
        }
        
        break;
      
      
      // NORMAL MODE (default)
      default:
      
        // Config pagination
        $total = $this->Ezpx->count_all($this->config->item('db_table_prefix') . 'photos');
        $start = $this->uri->segment(3, 0);
        
        // Get photos
        $photos = $this->Ezpx->get_all_photos($this->settings['photos_per_page'], $start);        
        $data['photos'] = $this->_standardize_photos_normal($photos->result_array());
        
        break;
    }
    
    
    // Build configuration for pagination
    $config['base_url'] = site_url('main/index');
    $config['total_rows'] = $total;
    $config['per_page'] = $this->settings['photos_per_page'];
    $config['uri_segment'] = 3;
    $config['first_link'] = '&laquo;&laquo; First';
    $config['last_link'] = 'Last &raquo;&raquo;';
    $config['next_link'] = '<strong>Next &raquo;</strong>';
    $config['prev_link'] = '<strong>&laquo; Previous</strong>';
    $config['num_links'] = 1;
        
    $this->pagination->initialize($config);
    
    
    // Get pagination data
    if ( ! $pager = $this->pagination->create_links_blog())
    {
      $pager['prev_link'] = '';
      $pager['next_link'] = '';
    }
    
    $data['prev_post_photo'] = '|';
    $data['next_post_photo'] = '|';
    
    // Generate previous and next post photo if in normal mode
    if ($this->settings['mode'] == 'NORMAL')
    {
      // Generate previous post photo
      $prev_photo = $this->Ezpx->get_prev_photo($start);
      if ($prev_photo->num_rows())
      {
        $prev_photo = $prev_photo->row_array();

        if (file_exists('./content/photos/tn_' . $prev_photo['photo_file']))
        {
          $data['prev_post_photo'] = '<img src="' . base_url() . '/content/photos/tn_' . $prev_photo['photo_file'] . '" border="0">|' . $pager['prev_link'];
        }      
      }

      // Generate next post photo
      $next_photo = $this->Ezpx->get_next_photo($start);
      if ($next_photo->num_rows())
      {
        $next_photo = $next_photo->result_array();
        $next_photo = $next_photo[0];

        if (file_exists('./content/photos/tn_' . $next_photo['photo_file']))
        {
          $data['next_post_photo'] = '<img src="' . base_url() . '/content/photos/tn_' . $next_photo['photo_file'] . '" border="0">|' . $pager['next_link'];
        }
      }
    }
    
    // Generate previous and next post links
    $data['prev_post_link'] = $pager['prev_link'];
    $data['next_post_link'] = $pager['next_link'];
    
    // Output
    $this->parser->parse('public/main', $data);
  }
  
  
  /**
  * Standardize photos for normal mode
  * 
  * Prepares photo array to proper display format.
  *
  * @access private
  * @param array
  * @return array
  */
  function _standardize_photos_normal($photos)
  {
    foreach($photos as $key => $value) 
    {
      // Dynamic resize is turned on
      if ($this->settings['dynamic_resize']) 
      {
        $photo_props = @getimagesize('./content/photos/' . $value['photo_file']);
      
        if ($this->settings['resize_to'])
        {
          $photos[$key]['photo_file'] = site_url('imageprocess/resize/' . $value['photo_file']);
          if ($photo_props[0] > $photo_props[1])
          {
            $photos[$key]['photo_height'] = calc_dim($photo_props[0], $photo_props[1], $this->settings['resize_to'], '');
            $photos[$key]['photo_width'] = $this->settings['resize_to'];
          }
          else
          {
            $photos[$key]['photo_width'] = calc_dim($photo_props[0], $photo_props[1], '', $this->settings['resize_to']);
            $photos[$key]['photo_height'] = $this->settings['resize_to'];
          }        
        }
      
        // Display the original photo
        else
        {
          $photos[$key]['photo_file'] = base_url() . 'content/photos/' . $value['photo_file'];
          $photos[$key]['photo_width'] = $photo_props[0];
          $photos[$key]['photo_height'] = $photo_props[1];
        }
      }
      
      // Dynamic resize is turned off
      else
      {
        if (file_exists('./content/photos/resized_' . $value['photo_file']))
        {
          $photos[$key]['photo_file'] = base_url() . 'content/photos/resized_' . $value['photo_file'];
          $photo_props = @getimagesize('./content/photos/resized_' . $value['photo_file']);
          $photos[$key]['photo_width'] = $photo_props[0];
          $photos[$key]['photo_height'] = $photo_props[1];
        }
        else
        {
          $photos[$key]['photo_file'] = base_url() . 'content/photos/' . $value['photo_file'];
          $photo_props = @getimagesize('./content/photos/' . $value['photo_file']);
          $photos[$key]['photo_width'] = $photo_props[0];
          $photos[$key]['photo_height'] = $photo_props[1];
        }
      }
      
      $photos[$key]['photo_details'] = site_url('photo/show/' . $value['id']);
      
    }
    
    return $photos;
  }
  
  
  
  /**
  * Standardize photos for flickr mode
  * 
  * Prepares photo array to proper display format.
  *
  * @access private
  * @param array
  * @return array
  */
  function _standardize_photos_flickr($photos)
  {
    $return = array();
    
    foreach($photos as $key => $value) 
    {
      $return[$key]['photo_file'] = $value['imgfile_url'];
      $return[$key]['photo_width'] = $value['sizes']['width'];
      $return[$key]['photo_height'] = $value['sizes']['height'];
      $return[$key]['photo_details'] = site_url('photo/show/' . $value['id']);
    }
    
    return $return;
  }
  
  
  function _get_flickr_photos($page)
  {
    // Get photos by photoset
    if ($this->flickr_settings['photoset_id'])
    {
      $photos = $this->flickr->get_set_photos($this->flickr_settings['photoset_id'], $page);
    }
    
    // Get all photos
    else
    {
      $photos = $this->flickr->get_photos($page);
    }
    
    if (is_array($photos))
    {
      foreach ($photos['photos']['photo'] as $key => $value) {

        // Get photo info
        $photo_info = $this->flickr->get_photo_info($value['id']);
        $photos['photos']['photo'][$key]['details'] = $photo_info;

        // Get photo sizes
        $photo_sizes = $this->flickr->get_photo_sizes($value['id']);

        // Get a specific size
        $display_size = $this->flickr_settings['display_size'] ? $this->flickr_settings['display_size'] : 'medium';
        $sizes = $this->flickr->get_size($photo_sizes, $display_size);
        $photos['photos']['photo'][$key]['sizes'] = $sizes;

        // Get photo url
        $photo_url = $this->flickr->get_photo_url($photo_sizes, $display_size);
        $photos['photos']['photo'][$key]['imgfile_url'] = $photo_url;
        
      }
    }
    
    return $photos;
  }
  
  
  
  // Flickr test
  function flickr_test()
  {
    // Get sets
    //$sets = $this->flickr->get_sets();
    
    // Get photos from set
    //$set_photos = $this->flickr->get_set_photos('72157621982004016', '1');
    
    // Get photos
    $photos = $this->flickr->get_photos(2);
    
    foreach ($photos['photos']['photo'] as $key => $value) {
      
      // Get photo info
      $photo_info = $this->flickr->get_photo_info($value['id']);
      
      // Get photo sizes
      $photo_sizes = $this->flickr->get_photo_sizes($value['id']);
      
      // Get a specific size
      $medium_sizes = $this->flickr->get_size($photo_sizes, 'medium');
      
      // Get photo url
      $photo_url = $this->flickr->get_photo_url($photo_sizes, 'medium');
      
      echo '<img src="' . $photo_url . '">';
      
      debug($medium_sizes);
            
    }
    
  }
  
}
?>