<?php
/** 
* Photo
*
* Displays photo details and commentform with captcha. Validate new comments and insert
* into database, then refresh page. 
* 
* @package      ezpx 
* @author       Peter Zimon 
*/ 
class Photo extends Controller {
  
  var $settings;


  function Photo() 
  {
    parent::Controller();
    
    $this->load->database();
    $this->load->model('Ezpx');
    
    $libraries = array('parser', 'pagination', 'form_validation', 'session');
    $this->load->library($libraries);
    
    $helpers = array('form');
    $this->load->helper($helpers);
    
    $plugins = array('captcha');
    $this->load->plugin($plugins);
    
    $stgs = $this->Ezpx->get_settings();
    $stgs = $stgs->result_array();
    $this->settings = $stgs[0];
    
    $this->settings['captcha_expiration'] = 7200;
    
    // Get FLICKR settings
    if ($this->settings['mode'] == 'FLICKR')
    {
      $this->flickr_settings = $this->Ezpx->get_flickr_settings();
      if ( ! $this->flickr_settings)
      {
        die('Your flickr settings are not complete. Please provide all necessary flickr information.');
      }
      $config['user_id'] = $this->flickr_settings['flickr_user_id'];
      $config['per_page'] = $this->settings['photos_per_page'];
      $this->load->library('flickr', $config);
    }
  }
  
  
  /** 
  * Default function
  *
  * This function is an alias of function show(). See there...
  * 
  * @access public
  */ 
  function index() 
  {
    $this->show();
  }
  
  
  /** 
  * Show
  *
  * Displays photo details (resized photo according to design, tags, commentform,
  * comments etc. Validatie new comments and insert comment if no errors occured.
  * 
  * @package      ezpx 
  */ 
  function show()
  {  
    $id = $this->uri->segment(3, 0);
    
    // Get global tags <- this must be called first for data displaying
    $data = $this->Ezpx->get_global_tags();
    
    // Get configuration
    $data['cfg_wrapper']   = $this->config->item('wrapper');
    $data['tpl_base_dir']  = 'content/templates/' . $this->config->item('template');
    $data['tpl_base_url']  = base_url() . 'content/templates/' . $this->config->item('template');
    
    // Get photo data depending on mode
    switch ($this->settings['mode'])
    {
      case 'FLICKR':
        // Get photo page data
        $append_data = $this->build_photopage_flickr($id);        
        $data = array_merge($data, $append_data);
        break;
        
      default:
        // Get photo page data
        $append_data = $this->build_photopage_normal($id);
        $data = array_merge($data, $append_data);
        break;
    }
    
    // Parse template
    $this->parser->parse('public/photo', $data);
  }
  
  
  /** 
  * Build photo page for normal mode
  * 
  * @access public
  * @param number
  * @return array
  */
  function build_photopage_normal($id)
  {
    $photo = $this->Ezpx->get_photo($id);
    $data = $photo->row_array();
    $photo_file = $data['photo_file'];
    
    // Get template config (for commentform it's needed again)
    $data['cfg_wrapper']   = $this->config->item('wrapper');
    $data['tpl_base_dir']  = 'content/templates/' . $this->config->item('template');
    $data['tpl_base_url']  = base_url() . 'content/templates/' . $this->config->item('template');
    
    // Display and resize photo dynamically
    $photo_props = @getimagesize('./content/photos/' . $photo_file);
    
    
    // Dynamic resize is turned on
    if ($this->settings['dynamic_resize'])
    {
      if ($this->settings['resize_to'])
      {
        $data['photo_file'] = site_url('imageprocess/resize/' . $photo_file);
        if ($photo_props[0] > $photo_props[1])
        {
          $data['photo_height'] = calc_dim($photo_props[0], $photo_props[1], $this->settings['resize_to'], '');
          $data['photo_width'] = $this->settings['resize_to'];
        }
        else
        {
          $data['photo_width'] = calc_dim($photo_props[0], $photo_props[1], '', $this->settings['resize_to']);
          $data['photo_height'] = $this->settings['resize_to'];
        }
        $data['original_file'] = base_url() . 'content/photos/' . $photo_file;
      }
      // Display the original photo
      else
      {
        $data['photo_file'] = base_url() . 'content/photos/' . $photo_file;
        $data['photo_width'] = $photo_props[0];
        $data['photo_height'] = $photo_props[1];
        $data['original_file'] = '';
      }
    
    // Dynamic resize is turned off
    } else {
      
      // Check photofile
      if (file_exists('./content/photos/resized_' . $data['photo_file']))
      {
        $photo_props = getimagesize('./content/photos/resized_' . $data['photo_file']);
        $data['photo_width'] = $photo_props[0];
        $data['photo_height'] = $photo_props[1];
        $data['original_file'] = base_url() . 'content/photos/' . $data['photo_file'];
        $data['photo_file'] = base_url() . 'content/photos/resized_' . $data['photo_file'];
      }
      else
      {
        $photo_props = @getimagesize('./content/photos/' . $data['photo_file']);
        $data['photo_width'] = $photo_props[0];
        $data['photo_height'] = $photo_props[1];
        $data['photo_file'] = base_url() . 'content/photos/' . $data['photo_file'];
        $data['original_file'] = '';
      }
    }
    
    // Clean up some vars
    if (trim($data['gallery_link']) == 'http://')
    {
      $data['gallery_link'] = '';
    }
    
    
    // Tags
    $tags = $this->Ezpx->get_tags($id);
    $all_tags = $tags->num_rows(); 
    $i = 0;
    $tags = $tags->result_array();
    $data['tags'] = '';
    foreach($tags as $tag)
    {
      $data['tags'] .= anchor('search/index/tag/' . base64_encode($tag['tag']), $tag['tag']);
      $i++;
      if ($i < $all_tags)
      {
        $data['tags'] .= ', ';
      }
    }
    
    // Commentform
    $data['comment_form'] = '';
    $data['comment_success'] = '';
    $data['comment_errors'] = '';
    $data['no_comments'] = '';
    
    if ($this->settings['comments_allowed'])
    { 
      // New comment validation configuration
      unset($config);
      $this->form_validation->set_rules('comment_name', $this->lang->line('name'), 'required');
      $this->form_validation->set_rules('comment_email', $this->lang->line('e_mail'), 'required|valid_email');
      $this->form_validation->set_rules('comment_body', $this->lang->line('comment_body'), 'required');
      $this->form_validation->set_rules('comment_captcha', $this->lang->line('captcha'), 'required|callback_captcha_check');
      
      // Validate new comment
      if ($this->form_validation->run() == TRUE) 
      {
        if ($this->Ezpx->insert_comment($this->input, $id))
        {
          $this->session->set_userdata('comment_name', $this->input->post('comment_name'));
          $this->session->set_userdata('comment_email', $this->input->post('comment_email'));
          redirect(site_url('photo/show/' . $id . '/comment_success#commentform'));
        }
        else
        {
          $data['comment_form'] .= 'An error occured during comment save.';
        }
      }
      else
      {
        $data['comment_errors'] = validation_errors();
      }
      
      if ($this->uri->segment(4) == 'comment_success')
      {
        $data['comment_success'] = $this->lang->line('comment_success');
      }
      
      // Generate captcha
      $vals = array(
				'img_path'	 => './captcha/',
				'img_url'	 => base_url() . '/captcha/',
				'font_path'	 => './system/fonts/Sansation_Bold.ttf',
				'img_width'	 => '200',
				'img_height' => '50',
				'expiration' => $this->settings['captcha_expiration']
			);
    	
    	$cap = create_captcha($vals);
    	
    	$cap_input = array(
				'captcha_id'	=> '',
				'captcha_time'	=> $cap['time'],
				'ip_address'	=> $this->input->ip_address(),
				'word'			=> $cap['word']
			);
    				
      $this->Ezpx->insert_captcha($cap_input);
      
      // Parse commentform      
      $data['comment_form_open'] = form_open(site_url('photo/show/' . $id . '#commentform'));
      $data['comment_form_close'] = form_close();
      $data['comment_name_input'] = '<input type="text" id="comment_name" name="comment_name" value="' . set_value('comment_name', $this->session->userdata('comment_name')) . '" />';
      $data['comment_email_input'] = '<input type="text" id="comment_email" name="comment_email" value="' . set_value('comment_email', $this->session->userdata('comment_email')) . '" />';
      $data['comment_body_input'] = '<textarea name="comment_body" id="comment_body">' . set_value('comment_body') . '</textarea>';
      $data['comment_captcha_image'] = $cap['image'];
      $data['comment_captcha_input'] = '<input type="text" name="comment_captcha" id="comment_captcha" value="" />';
      $data['comment_submit_button'] = '<input type="submit" id="comment_submit" name="go" value="Submit" />';
      
      $data['comment_form'] .= $this->parser->parse('public/commentform', $data, TRUE);
      
    }
    
    // Config comments pagination
    $per_page = 20;
    $start = (int)$this->uri->segment(4, 0);
    $total = $this->Ezpx->count_all_comments($id);
    
    $config['base_url'] = site_url('photo/show/' . $id);
    $config['total_rows'] = $total;
    $config['per_page'] = $per_page;
    $config['uri_segment'] = 4;
    $config['first_link'] = $this->lang->line('first');
    $config['last_link'] = $this->lang->line('last');
    $config['next_link'] = $this->lang->line('next');
    $config['prev_link'] = $this->lang->line('prev');
        
    $this->pagination->initialize($config);
    
    $data['comment_pager'] = $this->pagination->create_links();
    
    // Get comments
    $comments = $this->Ezpx->get_all_comments($id, $per_page, $start);
    $comments = $comments->result_array();
    
    if (count($comments))
    {
      foreach($comments as $comment)
      {
        $data['comments'][] = array(
          'comment_name' => $comment['name'],
          'comment_email' => $comment['email'],
          'comment_body' => $comment['body'],
          'comment_date' => $comment['post_date']
        );
      }
    }
    else
    {
      $data['comments'] = array();
      $data['no_comments'] = $this->lang->line('no_comments');
    }
    
    return $data;
  }
  
  
  /** 
  * Build photo page for normal mode
  * 
  * @access public
  * @param number
  * @return array
  */
  function build_photopage_flickr($id)
  {
    // Reset return
    $return = array();
    
    // Photo information
    $photo = $this->flickr->get_photo_info($id);
    if ( ! $photo)
      return FALSE;
    
    $return['title'] = $photo['photo']['title']['_content'];
    $return['body'] = $photo['photo']['description']['_content'];
    $return['upload_date'] = date('Y.m.d', $photo['photo']['dateuploaded']);
    
    // Tags
    $return['tags'] = '';
    foreach ($photo['photo']['tags']['tag'] as $tag)
    {
      $return['tags'] .= $tag['_content'] . ', ';
    }
    $return['tags'] = substr($return['tags'], 0, -2);
    
    // Photo sizes
    $photo_sizes = $this->flickr->get_photo_sizes($id);
    if ($photo_sizes)
    {
      $sizes = $this->flickr->get_size($photo_sizes, 'medium');
      if (isset($sizes['width']))
      {
        $return['photo_width'] = $sizes['width'];
        $return['photo_height'] = $sizes['height'];
      }
    }
    
    // Photo URL
    $return['photo_file'] = $this->flickr->get_photo_url($photo_sizes, 'medium');
    
    
    // Config comments pagination
    $per_page = 20;
    $start = (int)$this->uri->segment(4, 0);
    $total = (int)$photo['photo']['comments']['_content'];
    
    $config['base_url'] = site_url('photo/show/' . $id);
    $config['total_rows'] = $total;
    $config['per_page'] = $per_page;
    $config['uri_segment'] = 4;
    $config['first_link'] = $this->lang->line('first');
    $config['last_link'] = $this->lang->line('last');
    $config['next_link'] = $this->lang->line('next');
    $config['prev_link'] = $this->lang->line('prev');
        
    $this->pagination->initialize($config);
    
    $return['comment_pager'] = $this->pagination->create_links();
    
    
    // Comments
    $return['comments'] = array();
    $photo_page = $this->flickr->get_photo_page($photo['photo']['urls']['url']);
    $return['comment_form'] = '<div class="comments_external">Post comments to this photo on <a href="' . $photo_page . '" target="_blank">flickr</a>.</div>';
    $return['no_comments'] = '';
    $comments = $this->flickr->get_comments($id);
    if (isset($comments['comments']['comment']))
    {
      $chunk_comments = array_chunk($comments['comments']['comment'], $per_page);
      $page = round($start / $per_page);
      if (isset($chunk_comments[$page]))
      {
        foreach ($chunk_comments[$page] as $key => $value) {
          $return['comments'][$key]['comment_name'] = $value['authorname'];
          $return['comments'][$key]['comment_date'] = date('Y.m.d', $value['datecreate']);
          $return['comments'][$key]['comment_body'] = nl2br($value['_content']);
        }
      }
    }
    else
    {
      $return['no_comments'] = $this->lang->line('no_comments');
    }
    
    // Original file
    $return['original_file'] = $photo_page;
    
    return $return;
    
  }
  
  
  /**
  * Callback function for captcha check
  *
  * @access public
  * @param  string
  * @return boolean
  */
  function captcha_check($str)
  {
    $data['word'] = $str;
    $data['ip_address'] = $this->input->ip_address();
    $data['expiration'] = time() - $this->settings['captcha_expiration'];
    if ( ! $this->Ezpx->get_captcha($data))
    {
      $this->form_validation->set_message('captcha_check', $this->lang->line('captcha_error'));
      return FALSE;
    }
  }
  
}
?>