<?php
/** 
* Search
*
* Controller displays search results. Search can be executed by tag and by
* month.
* 
* @package      ezpx 
* @author       Peter Zimon 
*/ 
class Search extends Controller {

  var $settings;

  
  function Search() 
  {
    parent::Controller();
    
    $this->load->database();
    $this->load->model('Ezpx');
    
    $libraries = array('parser', 'pagination', 'form_validation');
    $this->load->library($libraries);
    
    $stgs = $this->Ezpx->get_settings();
    $this->settings = $stgs->row_array();;
    
    // Get FLICKR settings
    if ($this->settings['mode'] == 'FLICKR')
    {
      $this->flickr_settings = $this->Ezpx->get_flickr_settings();
      if ( ! $this->flickr_settings)
      {
        die('Your flickr settings are not complete. Please provide all necessary flickr information.');
      }
      $config['user_id'] = $this->flickr_settings['flickr_user_id'];
      $this->load->library('flickr', $config);
    }
  }
  
  
  /** 
  * Default function
  *
  * This function is an alias of function by(). See there...
  */ 
  function index()
  {
    $this->by();
  }
  
  
  /** 
  * Do the search
  *
  * Get search results
  * 
  * @access public
  */ 
  function by()
  {
    // Get global tags <- this must be called first for data displaying
    $data = $this->Ezpx->get_global_tags();
    
    // Get configuration
    $data['cfg_wrapper']   = $this->config->item('wrapper');
    $data['tpl_base_dir']  = 'content/templates/' . $this->config->item('template');
    $data['tpl_base_url']  = base_url() . 'content/templates/' . $this->config->item('template');
    
    // Config pagination
    $per_page = $this->settings['archive_per_page'];
    
    // Get search results
    $search_by = $this->uri->segment(3);
    
    // Switch search basis
    switch($search_by)
    {
      // Search by tag
      case 'tag':
        $uri_segment = 5;
        $start = $this->uri->segment($uri_segment, 0);
        $search_term = base64_decode($this->uri->segment(4));
        $data['search_title'] = $this->lang->line('search_by_tag') . ': ' . $search_term;
        $config['base_url'] = site_url('search/by/tag/' . base64_encode($search_term));
        
        switch ($this->settings['mode'])
        {
          case 'FLICKR':
            $start = ($start / $this->settings['archive_per_page']) + 1;
            $flickr_search = $this->_get_search_result_flickr($search_term, $start);
            $total = $flickr_search['total'];
            $data['search_results'] = $flickr_search['search_results'];
          
            break;
          default: 
            $total = $this->Ezpx->count_photos_by_tagname($search_term);
            $data['search_results'] = $this->_get_search_result_normal($search_term, $start);
            break;
        }
        break;
      
      // Search by month (not supported in flickr mode)
      case 'month':
        $uri_segment = 6;
        $start = $this->uri->segment($uri_segment, 0);
        $search_year = $this->uri->segment(4);
        $search_month = $this->uri->segment(5);
        $data['search_title'] = $this->lang->line('search_by_month') . ': ' . $search_year . '. ' . $search_month;
        $config['base_url'] = site_url('search/by/month/' . $search_year . '/' . $search_month);
        $total = $this->Ezpx->count_photos_by_month($search_year, $search_month);
        $data['search_results'] = $this->_get_search_monthly_normal($search_year, $search_month, $per_page, $start);
        break;
      
      // Do nothing if this var is not supplied 
      default:
        return FALSE;
        break;
    }
    
    $config['total_rows'] = $total;
    $config['per_page'] = $per_page;
    $config['uri_segment'] = $uri_segment;
    $config['first_link'] = $this->lang->line('first');
    $config['last_link'] = $this->lang->line('last');
    $config['next_link'] = $this->lang->line('next');
    $config['prev_link'] = $this->lang->line('prev');
    $config['num_links'] = 1;
        
    $this->pagination->initialize($config);
    
    // Get pagination data
    $data['search_pager'] = $this->pagination->create_links();
    
    // Output
    $this->parser->parse('public/search', $data);
  }
  
  
  /**
  * Return search results in normal mode
  *
  * @access private
  * @param string
  * @param number
  * @return string
  */
  function _get_search_result_normal($search_term, $start)
  {
    $photos = $this->Ezpx->get_photos_by_tagname($search_term, $this->settings['archive_per_page'], $start);
    $photos = $photos->result_array();
    
    $ret = '';
    foreach ($photos as $photo)
    {
      if (file_exists('./content/photos/tn_' . $photo['photo_file']))
      {
        $ret .= anchor('photo/show/' . $photo['id'], '<img src="' . base_url() . 'content/photos/tn_' . $photo['photo_file'] . '" border="0">');
      }
    }
    return $ret;
  }
  
  
  /**
  * Return search monthly in normal mode
  *
  * @access private
  * @param string
  * @param number
  * @return string
  */
  function _get_search_monthly_normal($search_year, $search_month, $per_page, $start)
  {
    $photos = $this->Ezpx->get_photos_by_month($search_year, $search_month, $per_page, $start);
    $photos = $photos->result_array();
    
    $ret = '';
    foreach ($photos as $photo)
    {
      if (file_exists('./content/photos/tn_' . $photo['photo_file']))
      {
        $ret .= anchor('photo/show/' . $photo['id'], '<img src="' . base_url() . 'content/photos/tn_' . $photo['photo_file'] . '" border="0">');
      }
    }
    return $ret;
  }
  
  
  /**
  * Return search results in flickr mode
  *
  * @access private
  * @param string
  * @param number
  * @return string
  */
  function _get_search_result_flickr($tag, $start)
  {
    $terms = array(
      'tags' => $tag
    );
    $photos = $this->flickr->search($start, $terms, $this->settings['archive_per_page']);
    
    $ret['total'] = $photos['photos']['total'];
    
    $ret['search_results'] = '';
    if (is_array($photos))
    {
      foreach ($photos['photos']['photo'] as $photo)
      {
        $ret['search_results'] .= anchor('photo/show/' . $photo['id'], '<img src="' . $this->flickr->lego_photo_url($photo, "Square") . '" border="0">');
      }
    }
    return $ret;
  }
}
?>