<?php
/** 
* Pagination
*
* Modified pagination to have pagination withoud digits. The blog style pagination
* is a reverse pagination since the latest entries should be displayed first.
* 
* @package      ezpx 
* @author       Peter Zimon based on Codeigniter pagination
*/ 
class EP_Pagination extends CI_Pagination {


  /** 
  * Create links blogstyle
  *
  * @access public
  */ 
  function create_links_blog()
  {
    // If our item count or per-page total is zero there is no need to continue.
		if ($this->total_rows == 0 OR $this->per_page == 0)
		{
		   return '';
		}

		// Calculate the total number of pages
		$num_pages = ceil($this->total_rows / $this->per_page);

		// Is there only one page? Hm... nothing more to do here then.
		if ($num_pages == 1)
		{
			return '';
		}

		// Determine the current page number.		
		$CI =& get_instance();
		
		if ($CI->config->item('enable_query_strings') === TRUE OR $this->page_query_string === TRUE)
		{
			if ($CI->input->get($this->query_string_segment) != 0)
			{
				$this->cur_page = $CI->input->get($this->query_string_segment);
				
				// Prep the current page - no funny business!
				$this->cur_page = (int) $this->cur_page;
			}
		}
		else
		{
			if ($CI->uri->segment($this->uri_segment) != 0)
			{
				$this->cur_page = $CI->uri->segment($this->uri_segment);
				
				// Prep the current page - no funny business!
				$this->cur_page = (int) $this->cur_page;
			}
		}

		$this->num_links = (int)$this->num_links;
		
		if ($this->num_links < 1)
		{
			show_error('Your number of links must be a positive number.');
		}
				
		if ( ! is_numeric($this->cur_page))
		{
			$this->cur_page = 0;
		}
		
		// Is the page number beyond the result range?
		// If so we show the last page
		if ($this->cur_page > $this->total_rows)
		{
			$this->cur_page = ($num_pages - 1) * $this->per_page;
		}
		
		$uri_page_number = $this->cur_page;
		$this->cur_page = floor(($this->cur_page/$this->per_page) + 1);

		// Calculate the start and end numbers. These determine
		// which number to start and end the digit links with
		$start = (($this->cur_page - $this->num_links) > 0) ? $this->cur_page - ($this->num_links - 1) : 1;
		$end   = (($this->cur_page + $this->num_links) < $num_pages) ? $this->cur_page + $this->num_links : $num_pages;

		// Is pagination being used over GET or POST?  If get, add a per_page query
		// string. If post, add a trailing slash to the base URL if needed
		if ($CI->config->item('enable_query_strings') === TRUE OR $this->page_query_string === TRUE)
		{
			$this->base_url = rtrim($this->base_url).'&amp;'.$this->query_string_segment.'=';
		}
		else
		{
			$this->base_url = rtrim($this->base_url, '/') .'/';
		}

  		// And here we go...
		$output = '';
		$first = '';
		$last = '';
		$prev = '';
		$next = '';
		$digits = '';
		$prev_link = '';
		$next_link = '';
		$last_link = '';
		$first_link = '';
		

		// Render the "First" link
		if  ($this->cur_page > ($this->num_links + 1))
		{
		  $first_link = $this->base_url;
			$first .= $this->last_tag_open.'<a href="'.$this->base_url.'">'.$this->last_link.'</a>'.$this->last_tag_close;
		}

		// Render the "previous" link
		if  ($this->cur_page != 1)
		{
			$i = $uri_page_number - $this->per_page;
			if ($i == 0) $i = '';
			$prev_link = $this->base_url.$i;
			$prev .= $this->next_tag_open.'<a href="'.$this->base_url.$i.'">'.$this->next_link.'</a>'.$this->next_tag_close;
		}

		// Render the "next" link
		if ($this->cur_page < $num_pages)
		{
		  $next_link = $this->base_url.($this->cur_page * $this->per_page);
			$next .= $this->prev_tag_open.'<a href="'.$this->base_url.($this->cur_page * $this->per_page).'">'.$this->prev_link.'</a>'.$this->prev_tag_close;
		}

		// Render the "Last" link
		if (($this->cur_page + $this->num_links) < $num_pages)
		{
			$i = (($num_pages * $this->per_page) - $this->per_page);
			$last_link = $this->base_url.$i;
			$last .= $this->first_tag_open.'<a href="'.$this->base_url.$i.'">'.$this->first_link.'</a>'.$this->first_tag_close;
		}
		
		// Concatenate output
		$output = $next . $prev;

		// Kill double slashes.  Note: Sometimes we can end up with a double slash
		// in the penultimate link so we'll kill all double slashes.
		$output = preg_replace("#([^:])//+#", "\\1/", $output);

		// Add the wrapper HTML if exists
		$output = $this->full_tag_open.$output.$this->full_tag_close;
		
		//return $output;
		$return['next_anchor'] = $prev;
		$return['prev_anchor'] = $next;
		$return['output'] = $output;
		$return['next_link'] = $prev_link;
		$return['prev_link'] = $next_link;
		
		return $return;
  }

}
?>