<?php
/** 
* Parser
*
* This library is based on Codeigniter's parser. It extends the functionality
* to be able to have parameters and rules to the parsed string.
* 
* @package      ezpx 
* @author       Peter Zimon based on Codeigniter's parser.
*/ 
class EP_Parser extends CI_Parser {

  /**
	* Apply the rules
	* 
	* All parsing rules are defined within this function.
	*
	* @access private
	* @param  array
	* @param  string
	* @param  array
	* @return string
	*/
	function _apply_rule($values, $key, $parameters)
	{
	  $output = '';
    switch($key)
    {
      // Previous post displaying a link
      case 'prev_post_link':
        if ($values[0])
        {
          $output .= ($parameters[0]) ? '<a href="' . $values[0] . '">' : '';
          $output .= $parameters[0];
          $output .= ($parameters[0]) ? '</a>' : '';
        }
        break;
      
      // Next post displaying a link
      case 'next_post_link':
        if ($values[0])
        {
          $output .= ($parameters[0]) ? '<a href="' . $values[0] . '">' : '';
          $output .= $parameters[0];
          $output .= ($parameters[0]) ? '</a>' : '';
        }
        break;
        
      // Previous post displaying a photo
      case 'prev_post_photo':
        if ($values[1])
        {
          $output .= ($parameters[0] == 'do_link') ? '<a href="' . $values[1] . '">' : '';
          $output .= $values[0];
          $output .= ($parameters[0] == 'do_link') ? '</a>' : '';
        }
        break;
      
      // Next post displaying a photo
      case 'next_post_photo':
        if ($values[1])
        {
          $output .= ($parameters[0] == 'do_link') ? '<a href="' . $values[1] . '">' : '';
          $output .= $values[0];
          $output .= ($parameters[0] == 'do_link') ? '</a>' : '';
        }
        break;
      
      // Photo width
      case 'photo_width':
        if ($values[0])
        {
          $output = $values[0] + (int)$parameters[0];
        }
        break;
      
      // Photo height
      case 'photo_height':
        if ($values[0])
        {
          $output = $values[0] + (int)$parameters[0];
        }
        break;
        
      // Default: needed if a parameter is applied to a key that has no parameters
      default:
        $output .= $values[0];
        break;
        
    }
    return $output;
	}
	
	
	
//---------------------------------------------------------------------------------

  /**
  * Overwrite standard parser to be able to use own rules
  * 
  * @access public
  * @param  string
  * @param  string
  * @param  string
  * @return string
  */
  function _parse_single($key, $val, $string)
	{ 
    $orig_val = $val;
    if ($matches = $this->_match_rules($key, $string))
    {
      foreach($matches[2] as $param_key => $param_string)
      {
        $parameters = explode("|", $param_string);
        $values = explode("|", $orig_val);
        
        $val = $this->_apply_rule($values, $key, $parameters);
        
        $string = str_replace($matches[0][$param_key], $val, $string);
      }
      
      return $string;
      
    }
    else
    {
      $values = explode("|", $val);
      
      return str_replace($this->l_delim.$key.$this->r_delim, $values[0], $string);	  
    }
	}
	

//---------------------------------------------------------------------------------
	
	/**
	* Check if rules are defined to the string
	* 
	* @access private
	* @param  string
	* @param  string
	* @return boolean/string
	*/
	function _match_rules($key, $string)
	{
      if (preg_match_all("|" . $this->l_delim . $key . "(\|)(.+?)" . $this->r_delim . "|s", $string, $out))
      {
        return $out;
      }
      else
      {
        return FALSE;
      }
	}

}
?>