<?php
class EZBrowser {
  
  var $config;
  var $error;
  var $dir;
  var $clear;
  var $language;
  
  
  /**
  * Constructor for PHP5
  ********************************************************************/
  function __construct($config)
  {
    global $_GET;
    global $_SESSION;
    
    // Set dir
    if ( ! $config['current_dir'])
    {
      if (isset($_GET['dir'])) 
      {
        $this->dir = $_GET['dir'];
      }
      else 
      {
        $this->dir = '';
      }
    }
    else
    {
      $this->dir = $config['current_dir'];
    }
    
    $this->dir = base64_decode($this->dir);
    
    // Clean up dir
    $this->dir = ereg_replace('\/+', '/', $this->dir);
    if (substr($this->dir, 0, 1) == '/') $this->dir = substr($this->dir, 1);
    if (substr($this->dir, -1, 1) != '/') $this->dir = $this->dir . '/';
    if (trim($this->dir == '/')) $this->dir = '';
    
    // Check session
    if ($config['session_check_variable'] !== FALSE)
    {
      if ( ! $_SESSION[$config['session_check_variable']])
      {
        die('You must be logged in to use the browser.');
      }
    }
    
    // @todo: Check whether base dir is writeable
    // if ( ! chmod($config['base_path'], '0777'))
    //     {
    //       die('Base folder is not writeable.');
    //     }
    
    // Set default config if needed
    $this->config = $config;
    $this->set_default_config();
    
    // Set language var
    $this->set_language();

    // Check configuration and exit if any mandatory missing
    $this->check_dirs();
    
    // Execute form check
    if ($this->form_check() === TRUE)
    {
      header($_SERVER['REQUEST_URI']);
    }
    
  }
  
  
  
  /*
  * Constructor for PHP4
  ********************************************************************/
  function EZBrowser($config)
  {
    $this->__construct($config);
  }
  
  
  
  /*
  * Set default configuration
  ********************************************************************/
  function set_default_config()
  {
    if ( ! isset($this->config['charset']))      $this->config['charset'] = 'UTF-8';
    if ( ! isset($this->config['language']))     $this->config['language'] = 'english';
    if ( ! isset($this->config['browser_url']))  $this->config['browser_url'] = 'index.php?dir=';
    $this->config['default_language'] = 'english';
  }
  
  
  
  /*
  * Set language var
  ********************************************************************/
  function set_language()
  {
    // English
    $language['english'] = array(
      'no_sub_folders' => 'No sub-folders.',
      'folder_empty' => 'This folder is empty.',
      'close' => 'Close',
      'create_folder' => 'Create folder',
      'new_folder_name' => 'New folder name',
      'ok' => 'Ok',
      'upload_file' => 'Upload file',
      'error_folder_spec_char' => 'Folder name cannot contain special characters.',
      'error_cant_create_folder' => 'Cannot create folder.',
      'error_folder_already_exists' => 'Folder already exists',
      'error_only_image_files_allowed' => 'Only image files are allowed.',
      'error_max_file_size_reached' => 'Maximum file size is reached. Please reduce the uploaded file\'s size.',
      'error_filecopy' => 'Error when copying file.',
      'home' => 'Home',
      'folders' => 'Folders'
    );
    
    // Hungarian
    $language['hungarian'] = array(
      'no_sub_folders' => 'Nincs alkönyvtár.',
      'folder_empty' => 'Ez a könyvtár üres.',
      'close' => 'Bezár',
      'create_folder' => 'Új könyvtár',
      'new_folder_name' => 'Új könyvtár neve',
      'ok' => 'Ok',
      'upload_file' => 'Fájl feltöltése',
      'error_folder_spec_char' => 'A könyvtár neve nem tartalmazhat speciális karaktereket.',
      'error_cant_create_folder' => 'Nem sikerült elkészíteni a könyvtárat.',
      'error_folder_already_exists' => 'A könyvtár már létezik',
      'error_only_image_files_allowed' => 'Csak képfájlok engedélyezettek.',
      'error_max_file_size_reached' => 'A fájl mérete meghaladja a maximálisan megengedettet. Kérlek csökkentsd a fájl méretét.',
      'error_filecopy' => 'Hiba a fájl másolása közben.',
      'home' => 'Kezdő könyvtár',
      'folders' => 'Könyvtárak'
    );
    
    if ( ! isset($language[$this->config['language']]))
    {
      $this->lang = $language[$this->config['default_language']];
    }
    else
    {
      $this->lang = $language[$this->config['language']];
    }    
  }
  
  
  
  /*
  * Check actual folders, base folder etc. Called from constructor
  ********************************************************************/
  function check_dirs()
  {
    if ( ! $this->config['base_path'])
    {
      die('Base path is not set or is not a folder!');
    }
    else
    {
      if ( ! is_dir($this->config['base_path']))
      {
        if ( ! @mkdir($this->config['base_path']))
        {
          die('Base folder' . $this->config['base_path'] . ' can\'t be created.');
        }
      }
    }
    
    // Check if current folder exists
    if ( ! is_dir($this->config['base_path'] . $this->dir))
    {
      die('Folder does not exist.');
    }
  }
  
  
  
  /*
  * Return folder tree
  ********************************************************************/
  function folders()
  {
    $output = '<div id="folders"><strong>' . $this->lang['folders'] . ':</strong><br />';
    
    $actual_dir = $this->config['base_path'] . $this->dir;
    
    // Read folder structure
    $handle = @opendir($actual_dir);
    
    // If handle is correct, read folders
    if ( ! $handle)
    {
      $output = 'Cannot read base folder.';
      return FALSE;
    }
    else
    {
      $subdirectories = '';
      $is_subdir = FALSE;
      while (false !== ($file = readdir($handle))) {
        if (is_dir($actual_dir . $file))
        {
          if (($file == '..' && $actual_dir != $this->config['base_path']) OR 
              ($file != '..' && $file != '.'))
          {
            if ($file != '..')
            {
              $to_dir = $this->dir . $file . '/';
              $is_subdir = TRUE;
            }
            else
            {
              $parent_dirs = explode('/', $this->dir);
              if (is_array($parent_dirs))
              {
                $to_dir = '';
                if (substr($this->dir, -1) == '/')
                {
                  $cnt = count($parent_dirs) - 2;
                }
                else
                {
                  $cnt = count($parent_dirs) - 1;
                }
                foreach ($parent_dirs as $key => $value) {
                  if ($key < $cnt && trim($value))
                  {
                    $to_dir .= $value . '/';
                  }
                }
              }
            }
            $subdirectories .= '<li><a href="' . $this->config['browser_url'] . base64_encode($to_dir) . '">' . $file . '</a></li>';
          }
        }
      }
      $output .= '<ul>';
      $output .= $subdirectories;
      $output .= '</ul>';
      
      if ( ! $is_subdir)
      {  
        $output .= '<div>' . $this->lang['no_sub_folders'] . '</div>';
      }

      closedir($handle);
    }
    
    $output .= '</div>';
    
    return $output;
  }
  
  
  
  /*
  * Return the contents of a folder
  ********************************************************************/
  function contents()
  {
    // 
    $output = '<div id="contents">';
    
    // Display error
    if ($this->error)
    {
      $output .= '
      <div class="error">
      ' . $this->error . '
      </div>
      ';
    }
    
    // Set default actual directory
    $full_dir = $this->config['base_path'] . $this->dir;
    $full_url = $this->config['base_url'] . $this->dir;
    
    // Read folder structure
    $handle = @opendir($full_dir);
    
    // If handle is correct, read folders
    if ( ! $handle)
    {
      $output .= 'Cannot read base folder.';
      return FALSE;
    }
    else
    {
      $files = '';
      $is_files = FALSE;
      while (false !== ($file = readdir($handle))) {
        if ( ! is_dir($full_dir . $file))
        {
          $img_props = @getimagesize($full_dir . $file);
          if ($img_props[0] > $img_props[1])
          {
            $dimension = ' width="130"';
          }
          else
          {
            $dimension = ' height="130"';
          }
          $is_files = TRUE;
          $this->clear = TRUE;
          $content = '<img src="' . $full_url . $file . '" ' . $dimension . ' /><br />';
          $content .= $file;
          $files .= '<li><a href="javascript:;" onClick="javascript:SelectFile( \'' . $full_url . $file . '\' );">' . $content . '</a></li>';
        }
      }
      if ($is_files == TRUE)
      {
        $output .= '<ul>';
        $output .= $files;
        $output .= '</ul>';
      }
      else
      {
        $output .= '<div>' . $this->lang['folder_empty'] . '</div>';
      }
      
    }
    
    $output .= '</div><br class="clearboth" />';
    
    return $output;
  }
  
  
  
  /*
  * Create Folder form
  ********************************************************************/
  function create_folder()
  {
    global $_POST;
    
    $new_dir = '';
    if (isset($_POST['new_dir'])) $new_dir = $_POST['new_dir'];
    
    $output = '
    <div class="dialog_container" id="create_folder">
      <div class="dialog">
      <div class="floatright"><a href="javascript:;" onClick="javascript:showCreateFolder(0);">' . $this->lang['close'] . '</a></div>
      <h2>' . $this->lang['create_folder'] . '</h2>
      <form name="upload_file" method="POST" enctype="multipart/form-data">
      <div class="margin_b">
        ' . $this->lang['new_folder_name'] . ':
        <input name="new_dir" type="input" size="30" class="inputfield" value="' . $new_dir . '" />
      </div>
      <div class="submit_container">
        <input name="sbmt" value="' . $this->lang['ok'] . '" type="submit" />
      </div>      
      <input type="hidden" name="current_dir" value="' . $this->dir . '">
      <input type="hidden" name="go" value="1">
      </form>
      </div>
    </div>
    ';
    return $output;
  }
  
  
  
  /*
  * Upload file form
  ********************************************************************/
  function upload_file()
  {
    $output = '
    <div id="upload_file" class="dialog_container">
      <div class="dialog">
      <div class="floatright"><a href="javascript:;" onClick="javascript:showUploadFile(0);">' . $this->lang['close'] . '</a></div>
      <h2>' . $this->lang['upload_file'] . '</h2>
      <form name="upload_file" method="POST" enctype="multipart/form-data">
      <div class="margin_b">
        ' . $this->lang['upload_file'] . ': <input name="new_file" type="file" />
      </div>
      <div class="submit_container">
        <input name="sbmt" value="' . $this->lang['ok'] . '" type="submit" />
      </div>      
      <input type="hidden" name="current_dir" value="' . $this->dir . '">
      <input type="hidden" name="go" value="1">
      </form>
      </div>
    </div>
    ';
    return $output;
  }
  
  
  /*
  * Css
  ********************************************************************/
  function css()
  {
    $output = '
    * {
      margin: 0;
      padding: 0;
    }

    body {
      font-family: "Lucida Grande", Arial, helvetica, sans-serif;
      font-size: 13px;
      color: #676767;
      margin: 20px;
    }

    input {
      font-size: 1em;
    }

    a {
      color: #33C;
    }

    #folders {
      float: left;
      width: 220px;
      height: 95%;
      overflow: auto;
      padding: 10px;
      background-color: #DDD;
    }

    #folders ul {
      list-style: none;
      list-style-type: none;
    }

    #folders ul li {
      margin-bottom: 5px;
    }

    #contents {
      margin-left: 260px;
      margin-top: 20px;
    }

    .margin_b {
      margin-bottom: 5px;
    }

    #contents ul {
      list-style: none;
      list-style-type: none;
    }

    #contents ul li {
      margin-bottom: 10px;
      margin-right: 10px;
      float: left;
      width: 130px;
      height: 150px;
      border: 1px solid #DDD;
      padding: 5px;
      text-align: center;
      font-size: 0.85em;
    }

    #contents ul li a {
      text-decoration: none;
      color: #676767;
    }

    #contents ul li img {
      border: 0;
    }

    h1 {
      font-size: 1.7em;
      margin-bottom: 10px;
    }

    h2 {
      font-size: 1.4em;
      margin-bottom: 10px;
    }

    .breadcrumbs {
      border-bottom: 1px solid #DDD;
      padding: 5px;
      margin-left: 260px;
      font-size: 0.85em;
    }

    .breadcrumbs a {
      text-decoration: none;
    }

    .form_label {
      float: left;
      width: 140px;
      padding-bottom: 5px;
    }

    .form_input {
      padding-bottom: 5px;
    }

    .clearboth {
      clear: both;
    }

    .floatright {
      float: right;
    }

    .dialog_container {
      visibility: hidden;
      position: absolute;
      overflow: hidden;
      left: 0;
      top: 0;
      width: 100%;
      height: 100%;
      z-index: 100;
    }

    .dialog {
      z-index: 102;
      background-color: #EFEFEF;
      border: 3px solid #999;
      padding: 20px;
      width: 400px;
      margin: 20px auto;
      filter: alpha(opacity=100); /* IE */
      opacity: 1.00; /* Safari, Opera and Mozilla */
    }

    .submit_container {
      padding-top: 10px;
      text-align: center;
    }

    .error {
      margin-bottom: 20px;
      background-color: #FCC;
      border: 1px solid #A00;
      color: #A00;
      padding: 10px;
    }
    ';
    return $output;
  }
  
  
  
  /*
  * HTML header
  ********************************************************************/
  function header()
  {
    $output = '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Frameset//EN"
       "http://www.w3.org/TR/html4/frameset.dtd">
    <html>
    <head>
    <title>EZ Browser</title>
    <style type="text/css">
    ' . $this->css() . '
    </style>
    <!--script type="text/javascript" src="http://jqueryjs.googlecode.com/files/jquery-1.3.2.min.js"></script-->
    <script type="text/javascript">    
      function SelectFile( fileUrl )
      {
        // window.opener.SetUrl( url, width, height, alt);
        window.opener.SetUrl( fileUrl ) ;
        window.close() ;
      }
      
      function showCreateFolder(show)
      {
        cF = document.getElementById(\'create_folder\');
        if (show)
        {
          cF.style.visibility = \'visible\';
          showUploadFile(0);
        }
        else
        {
          cF.style.visibility = \'hidden\';
        }
      }
      
      function showUploadFile(show)
      {
        uF = document.getElementById(\'upload_file\');
        if (show)
        {
          uF.style.visibility = \'visible\';
          showCreateFolder(0);
        }
        else
        {
          uF.style.visibility = \'hidden\';
        }
      }
    </script>
    </head>
    <body>
    <div>
      <h1>EZ Browser</h1>
    </div>
    ';
    
    return $output;
  }
  
  
  
  /*
  * HTML footer
  ********************************************************************/
  function footer()
  {
    $output = '';
    $output .= '
      </div>

    </body>
    </html>
    ';
    
    return $output;
  }
  
  
  
  /*
  * Form check
  ********************************************************************/
  function form_check()
  {
    global $_POST;
    global $_FILES;
    
    if (isset($_POST['go']))
    {
      // Create folder
      if (isset($_POST['new_dir'])) 
      {
        if ($_POST['new_dir'])
        {
          if ( ! ereg("^([0-9a-zA-Z_]+)$", $_POST['new_dir'])) {
            $this->error .= $this->lang['error_folder_spec_char'] . '<br />';
      			return false;
      		}
        
          if ( ! is_dir($this->config['base_path'] . $_POST['current_dir'] . $_POST['new_dir']))
          {
            if ( ! mkdir($this->config['base_path'] . $_POST['current_dir'] . $_POST['new_dir']))
            {
              $this->error .= $this->lang['error_cant_create_folder'] . '<br />';
              return FALSE;
            }
          }
          else
          {
            $this->error .= $this->lang['error_folder_already_exists'] . '(' . $_POST['new_dir'] . ')<br />';
            return FALSE;
          }
        }
      }
      // Upload file
      if (isset($_FILES['new_file']['tmp_name']))
      {
        // Check if it's an image
        if (strpos($_FILES['new_file']['type'], 'image') === FALSE)
        {
          $this->error .= $this->lang['error_only_image_files_allowed'] . '<br />';
          return FALSE;
        }
        
        // Check filesize
        if ($_FILES['new_file']['size'] > $this->config['max_file_size'])
        {
          $this->error .= $this->lang['error_max_file_size_reached'] . '<br />';
          return FALSE;
        }
        
        // Upload file
        if ( ! move_uploaded_file($_FILES['new_file']['tmp_name'], $this->config['base_path'] . $this->dir . $_FILES['new_file']['name']))
        {
          $this->error .= $this->lang['error_filecopy'] . '<br />';
          return FALSE;
        }
      }
      return TRUE;
    }
    
    return FALSE;    
  }
  
  
  /*
  * Breadcrumbs
  ********************************************************************/
  function breadcrumbs()
  {
    $output = '<div class="breadcrumbs">';

    $output .= '
      <div class="floatright">
        <a href="javascript:;" onClick="javascript:showCreateFolder(1);">' . $this->lang['create_folder'] . '</a> | 
        <a href="javascript:;" onClick="javascript:showUploadFile(1);">' . $this->lang['upload_file'] . '</a>
      </div>
    ';
    
    $dirs = explode('/', $this->dir);
    
    $output .= '<a href="' . $this->config['browser_url'] . '">' . $this->lang['home'] . '</a> ';
    
    if (is_array($dirs))
    {
      $prev_dir = '';
      foreach ($dirs as $key => $value) {
        if (trim($value))
        {
          $output .= ' &raquo; <a href="' . $this->config['browser_url'] . base64_encode($prev_dir . $value) . '/">' . $value . '</a>';
          $prev_dir .= $value . '/';
        }
      }
    }
    
    $output .= '</div>';
    
    return $output;
  }
  
  
  
  /*
  * Parse output
  ********************************************************************/
  function parse()
  {
    $output  = '';
    $output .= $this->header();
    $output .= $this->create_folder();
    $output .= $this->upload_file();
    $output .= $this->folders();
    $output .= $this->breadcrumbs();
    $output .= $this->contents();
    $output .= $this->footer();
    
    return $output;
  }
  
  
  /*
  * Display parsed output
  ********************************************************************/
  function display()
  {
    header('Content-Type:text/html; charset=' . $this->config['charset']);
    $output = $this->parse();
    echo $output;
  }
  
  
  
  
  
  
  /*******************************************************************
  * HELPER FUNCTIONS
  *******************************************************************/
  
  // Debug
  function debug($var)
  {
    echo '<pre>';
    print_r($var);
    echo '</pre>';
  }
  
}
?>