<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed'); 

class Flickr {
  
  var $api_key;
  var $user_id;
  var $url;
  var $per_page;
  
  
  function Flickr($config)
  {
    if ( ! isset($config['user_id']))
    {
      die('Your flickr user id is not set. Please provide it in EZPX settings.');
    }
    $this->api_key = 'f496d59af172738274a589f39a84a85c';
    $this->user_id = $config['user_id'];
    $this->url = "http://api.flickr.com/services/rest/?";
    $this->per_page = (isset($config['per_page'])) ? $config['per_page'] : 1;
  }
  
  
  /**
  * Get photos
  * 
  * Retrieves all photos specific to the user. 
  *
  * @access public
  * @param  number
  * @return array
  */
  function get_photos($page)
  {
    $params = array(
    	'api_key'	=> $this->api_key,
    	'user_id' => $this->user_id,
    	'per_page' => $this->per_page,
    	'page' => $page,
    	'method'	=> 'flickr.photos.search',
    	'format'	=> 'php_serial'
    );

    // Call flickr
    if ( ! $rsp_obj = $this->_call_flickr($params))
    {
      return FALSE;
    }
    
    // Return response
    if ( ! $this->_check_response($rsp_obj))
    {
      return FALSE;
    }
    else
    {
      return $rsp_obj;
    }    
  }
  
  
  /**
  * Search photos
  * 
  * Retrieves all photos specific to the user. 
  *
  * @access public
  * @param  number
  * @return array
  */
  function search($page, $search_terms, $per_page)
  {
    $params = array(
    	'api_key'	=> $this->api_key,
    	'user_id' => $this->user_id,
    	'per_page' => $per_page,
    	'page' => $page,
    	'method'	=> 'flickr.photos.search',
    	'format'	=> 'php_serial'
    );
    
    if (is_array($search_terms))
    {
      $params = array_merge($params, $search_terms);
    }

    // Call flickr
    if ( ! $rsp_obj = $this->_call_flickr($params))
    {
      return FALSE;
    }
    
    // Return response
    if ( ! $this->_check_response($rsp_obj))
    {
      return FALSE;
    }
    else
    {
      return $rsp_obj;
    }    
  }
  
  
  /**
  * Get sets
  * 
  * Retrieves all sets specific to the user. 
  *
  * @access public
  * @return array
  */
  function get_sets()
  {
    $params = array(
    	'api_key'	=> $this->api_key,
    	'user_id' => $this->user_id,
    	'method'	=> 'flickr.photosets.getList',
    	'format'	=> 'php_serial'
    );

    // Call flickr
    if ( ! $rsp_obj = $this->_call_flickr($params))
    {
      return FALSE;
    }
    
    // Return response
    if ( ! $this->_check_response($rsp_obj))
    {
      return FALSE;
    }
    else
    {
      return $rsp_obj;
    }
  }
  
  
  /**
  * Get set photos
  * 
  * Retrieves all photos specific to a set. 
  *
  * @access public
  * @param  number
  * @param  number
  * @return array
  */
  function get_set_photos($photoset_id, $page = 1)
  {
    $params = array(
    	'api_key'	    => $this->api_key,
    	'photoset_id' => $photoset_id,
    	'method'	    => 'flickr.photosets.getPhotos',
    	'per_page'    => $this->per_page,
    	'page'        => $page,
    	'format'      => 'php_serial'
    );

    // Call flickr
    if ( ! $rsp_obj = $this->_call_flickr($params))
    {
      return FALSE;
    }
    
    // Return response
    if ( ! $this->_check_response($rsp_obj))
    {
      return FALSE;
    }
    else
    {
      return $rsp_obj;
    }
  }
  
  
  /**
  * Get photo info
  * 
  * Returns photo information
  *
  * @access public
  * @param  number
  * @return array
  */
  function get_photo_info($photo_id)
  {
    $params = array(
    	'api_key'	    => $this->api_key,
    	'photo_id'    => $photo_id,
    	'method'	    => 'flickr.photos.getInfo',
    	'format'      => 'php_serial'
    );

    // Call flickr
    if ( ! $rsp_obj = $this->_call_flickr($params))
    {
      return FALSE;
    }
    
    // Return response
    if ( ! $this->_check_response($rsp_obj))
    {
      return FALSE;
    }
    else
    {
      return $rsp_obj;
    }
  }
  
  
  /**
  * Get photo sizes
  * 
  * Returns size information of a photo
  *
  * @access public
  * @param  number
  * @return array
  */
  function get_photo_sizes($photo_id)
  {
    $params = array(
    	'api_key'	    => $this->api_key,
    	'photo_id'    => $photo_id,
    	'method'	    => 'flickr.photos.getSizes',
    	'format'      => 'php_serial'
    );

    // Call flickr
    if ( ! $rsp_obj = $this->_call_flickr($params))
    {
      return FALSE;
    }
    
    // Return response
    if ( ! $this->_check_response($rsp_obj))
    {
      return FALSE;
    }
    else
    {
      return $rsp_obj;
    }
  }
  
  
  /**
  * Get comments of a photo
  * 
  * Returns comments of a photo
  *
  * @access public
  * @param  number
  * @return array
  */
  function get_comments($photo_id)
  {
    $params = array(
    	'api_key'	    => $this->api_key,
    	'photo_id'    => $photo_id,
    	'method'	    => 'flickr.photos.comments.getList',
    	'format'      => 'php_serial'
    );

    // Call flickr
    if ( ! $rsp_obj = $this->_call_flickr($params))
    {
      return FALSE;
    }
    
    // Return response
    if ( ! $this->_check_response($rsp_obj))
    {
      return FALSE;
    }
    else
    {
      return $rsp_obj;
    }
  }
  
  
  /**
  * Get photo url
  * 
  * Returns photo URL to a photo. First parameter can either be
  * a photo id or a photo sizes array (see method $this->get_photo_sizes())
  *
  * @access public
  * @param  array or number
  * @param  char
  * @return array
  */
  function get_photo_url($photo_input, $size = 'Medium')
  {
    // $photo can be a photo id or a photo_sizes array
    if (is_array($photo_input))
    {
      $photo_sizes = $photo_input;
    }
    else
    {
      $photo_sizes = $this->get_photo_sizes($photo_input);
      if ( ! is_array($photo_sizes))
        return FALSE;
    }
    
    if ( ! isset($photo_sizes['sizes']['size']))
      return FALSE;
    
    if (is_array($photo_sizes['sizes']['size']))
    {
      foreach ($photo_sizes['sizes']['size'] as $key => $value) {
        if (strtolower($value['label']) == strtolower($size))
        {
          return $value['source'];
        }
      }
    }
  }
  
  
  /**
  * Get photo page
  * 
  * Returns photo flickr page of a photo.
  *
  * @access public
  * @param  array
  * @return string
  */
  function get_photo_page($photo_urls)
  {
    foreach ($photo_urls as $url) {
      if ($url['type'] == 'photopage')
        return $url['_content'];
    }
  }
  
  
  /**
  * Get photo url
  * 
  * Returns the exact size of a photo to a given sizing. First parameter can 
  * either be a photo id or a photo sizes array (see method $this->get_photo_sizes())
  *
  * @access public
  * @param  array or number
  * @param  char
  * @return array
  */
  function get_size($photo_input, $size = 'Medium')
  {
    // $photo can be a photo id or a photo_sizes array
    if (is_array($photo_input))
    {
      $photo_sizes = $photo_input;
    }
    else
    {
      $photo_sizes = $this->get_photo_sizes($photo_input);
      if ( ! is_array($photo_sizes))
        return FALSE;
    }
    
    if ( ! isset($photo_sizes['sizes']['size']))
      return FALSE;
    
    if (is_array($photo_sizes['sizes']['size']))
    {
      foreach ($photo_sizes['sizes']['size'] as $key => $value) {
        if (strtolower($value['label']) == strtolower($size))
        {
          $return['width'] = $value['width'];
          $return['height'] = $value['height'];
        }
      }
    }
    
    return $return;
  }
  
  
  /**
  * Get recent photos
  *
  * Returns photos in an archive style
  *
  * @access public
  * @param  number
  * @param  char
  * @return array
  */
  function get_recent($page, $per_page)
  {
    $params = array(
    	'api_key'	=> $this->api_key,
    	'user_id' => $this->user_id,
    	'per_page' => $per_page,
    	'page' => $page,
    	'method'	=> 'flickr.photos.search',
    	'format'	=> 'php_serial'
    );

    // Call flickr
    if ( ! $rsp_obj = $this->_call_flickr($params))
    {
      return FALSE;
    }
    
    // Return response
    if ( ! $this->_check_response($rsp_obj))
    {
      return FALSE;
    }
    else
    {
      $ret['total'] = $rsp_obj['photos']['total'];
      
      foreach ($rsp_obj['photos']['photo'] as $key => $photo) {
        $ret['photos'][$key]['id'] = $photo['id'];
        $ret['photos'][$key]['photo_file'] = $this->lego_photo_url($photo, "Square");
      }
      return $ret;
    }
  }
  
  
  /**
  * Get tags
  *
  * Returns tags
  *
  * @access public
  * @param  number
  * @param  char
  * @return array
  */
  function get_tags()
  {
    $params = array(
    	'api_key'	=> $this->api_key,
    	'user_id' => $this->user_id,
    	'method'	=> 'flickr.tags.getListUserPopular',
    	'count'   => '9999',
    	'format'	=> 'php_serial'
    );

    // Call flickr
    if ( ! $rsp_obj = $this->_call_flickr($params))
    {
      return FALSE;
    }
    
    // Return response
    if ( ! $this->_check_response($rsp_obj))
    {
      return FALSE;
    }
    else
    { 
      return $rsp_obj;
    }
  }
  
  
  /**
  * Builds photo url 
  *
  * Returns photos in an archive style
  *
  * @access public
  * @param  number
  * @param  char
  * @return array
  */
  function lego_photo_url ($photo, $size = "Medium")
  {
    $sizes = array(
        "square" => "_s",
        "thumbnail" => "_t",
        "small" => "_m",
        "medium" => "",
        "large" => "_b"
    );
    
    $size = strtolower($size);
    
    if (!array_key_exists($size, $sizes)) {
      $size = "medium";
    }
    
    $url = "http://farm" . $photo['farm'] . ".static.flickr.com/" . $photo['server'] . "/" . $photo['id'] . "_" . $photo['secret'] . $sizes[$size] . ".jpg";
    
    return $url;
  }
  
  
  /**
  * Call flickr
  * 
  * Private method that builds flickr method to a php array.
  *
  * @access private
  * @param  array
  * @return array
  */
  function _call_flickr($params)
  {
    $encoded_params = array();
    foreach ($params as $k => $v){
    	$encoded_params[] = urlencode($k).'='.urlencode($v);
    }

    // Call the API and decode the response
    $url = $this->url . implode('&', $encoded_params);
    if ($rsp = @file_get_contents($url))
    {
      if ( ! $rsp)
      {
        return FALSE;
      }
      $rsp_obj = unserialize($rsp);
      return $rsp_obj;
    }
    else
    {
      return FALSE;
    }
  }
  
  
  /**
  * Check response
  * 
  * Returns false if the response got back with error.
  *
  * @access private
  * @param  array
  * @return boolean
  */
  function _check_response($obj)
  {
    if ( ! isset($obj['stat']))
      return FALSE;
      
    switch ($obj['stat']) {
      case 'ok':
        return TRUE;
        break;
      
      default:
        return FALSE;
        break;
    }
  } 
}
?>