<?php
/** 
* ezpx model
*
* This is the main model class for ezpx. One can retrieve any necessary database
* records using the methods. The model contains only DML queries, for DDL the 
* model Installer has to be used.
* 
* @package      ezpx 
* @author       Peter Zimon 
*/ 
class Ezpx extends Model {

  var $prefix;

  function Ezpx()
  {
    parent::Model();
    $this->prefix = $this->config->item('db_table_prefix');
  }
  
  
  /**
  * GLOBAL: Get all records from table
  * 
  * @access public
  * @param  string
  * @return array
  */
  function get_all($table)
  {
    $query = $this->get($table);
    return $query->result_array();
  }
  
  
  /**
  * GLOBAL: Get all records from table
  * 
  * @access public
  * @param  string
  * @param  string
  * @return array
  */
  function get_one($table, $id)
  {
    $this->db->where('id', $id);
    $query = $this->get($table);
    return $query->row_array();
  }

  
  /**
  * PHOTO: Insert new photo
  * 
  * @access public
  * @param  string
  * @param  array
  * @param  object
  * @return boolean
  */
  function insert_photo($photo_file, $tags, $input)
  {
    $sql = "
		 INSERT INTO ".$this->prefix."photos 
		         SET title = ". $this->db->escape($input->post('title')) .",
		             photo_file = ". $this->db->escape($photo_file) .",
		             body = " . $this->db->escape($input->post('body')) . ",
		             upload_date = ". $this->db->escape(ep_convert_date_to_db($input->post('post_date'), $this->lang->line('today'))) .",
		             gallery_link = ". $this->db->escape($input->post('gallery_link')) .",
		             post_date = NOW()
		";
		
		if ( ! $this->db->query($sql))
		{
		  return FALSE;
		}
		
		// Tags
	  if (is_array($tags))
	  {
	    $photo_id = $this->db->insert_id();
	    $return = TRUE;
	    
	    foreach($tags as $tag)
	    {
	      $tag = trim($tag);
	      
	      if ($tag != '')
	      {
		      $sql = "
		        INSERT INTO ".$this->prefix."tags
		                SET photo_id = ". $this->db->escape($photo_id) .",
		                    tag = ". $this->db->escape($tag) ."
		      ";
		      
		      if ( !  $this->db->query($sql))
		      {
		        $return = FALSE;
		        break;
		      }
	      }		      
	    }
	  }
	  
	  return $return;
  }
  
  
  /**
  * PHOTO: Update photo
  * 
  * @access public
  * @param  string
  * @param  array
  * @param  object
  * @return boolean
  */
  function update_photo($id, $tags, $input)
  {
    $upload_date = $input->post('post_year') . '-' . $input->post('post_month') . '-' . $input->post('post_day');
    $sql = "
		 UPDATE ".$this->prefix."photos 
		    SET title = ". $this->db->escape($input->post('title')) .",
		        body = " . $this->db->escape($input->post('body')) . ",
		        upload_date = ". $this->db->escape(ep_convert_date_to_db($input->post('post_date'), $this->lang->line('today'))) .",
		        gallery_link = ". $this->db->escape($input->post('gallery_link')) .",
		        post_date = NOW()
		  WHERE id = ". $this->db->escape($id) ."
		";
		if ( ! $this->db->query($sql))
		{
		  return false;
		}
		
		// Tags
	  if (is_array($tags))
	  {
	    $return = TRUE;
	    
	    if ( ! $this->delete_tags($id))
	    {
	      return FALSE;
	    }
	    
	    foreach($tags as $tag)
	    {
	      $tag = trim($tag);
	      
	      if ($tag != '')
	      {
		      $sql = "
		        INSERT INTO ".$this->prefix."tags
		                SET photo_id = ". $this->db->escape($id) .",
		                    tag = ". $this->db->escape($tag) ."
		      ";
		      
		      if ( !  $this->db->query($sql))
		      {
		        $return = FALSE;
		        break;
		      }
	      }		      
	    }
	  }
	  
	  return $return;
  }
  
  
  /**
  * PHOTO: Retrieve all photos
  * 
  * @access public
  * @param  string
  * @param  string
  * @return object
  */
  function get_all_photos($offset = '', $start = '')
  {
    $this->db->order_by('upload_date DESC, id DESC');
    if ($offset != '') 
    {
      return $this->db->get($this->prefix . 'photos', $offset, $start);
    }
    else
    {
      return $this->db->get($this->prefix .'photos');
    }
  }
  
  
  /**
  * GLOBAL: Count all records in a table
  * 
  * @access public
  * @param  string
  * @return object
  */
  function count_all($table_name)
  {
    return $this->db->count_all($table_name);
  }
  
  
  /**
  * PHOTO: Delete photo
  * 
  * @access public
  * @param  string
  * @return boolean
  */
  function delete_photo($id)
  {
    if ($this->db->delete($this->prefix . 'photos', array('id' => $id )))
    {
      return TRUE;
    }
    return FALSE;
  }
  
  
  /**
  * PHOTO: Retrun one photo
  * 
  * @access public
  * @param  string
  * @return object
  */
  function get_photo($id)
  {
    return $this->db->get_where($this->prefix . 'photos', array('id' => $id));
  }
  
  
  /**
  * SETTINGS: Check settings
  *
  * Check if any record exists in settings table
  * 
  * @access public
  * @return integer
  */
  function check_settings()
  {
    return $this->db->count_all($this->prefix . 'settings');
  }
  
  
  /**
  * SETTINGS: Get settings
  *
  * Get settings and insert default row if table is empty
  * 
  * @access public
  * @return object
  */
  function get_settings()
  {
    $result = $this->db->get($this->prefix . 'settings');
    
    if ($result->num_rows()) 
    {
      return $result;
    }
    else
    {
      // Insert default settings to global settings
      $defaults = array(
        'photos_per_page' => '1',
        'archive_per_page' => '10',
        'dynamic_resize' => '0',
        'resize_to' => '650',
        'thumbnail_width' => '120',
        'thumbnail_height' => '120',
        'comments_allowed' => '1',
        'blog_title' => 'My photoblog',
        'display_menu_photos' => '1',
        'display_menu_archives' => '1',
        'display_menu_links' => '1',
        'display_menu_pages' => '1',
        'photo_view' => 'LIST',
        'mode' => 'NORMAL',
        'rss' => '1'
      );
      $this->db->insert($this->prefix . 'settings', $defaults);      
      return $this->db->get($this->prefix . 'settings');
    }
  }
  
  
  /**
  * SETTINGS: Update global settings
  * 
  * @access public
  * @param  string
  * @return boolean
  */
  function update_settings($input)
  {
    // Global settings
    $sql = "
      UPDATE ".$this->prefix."settings
         SET photos_per_page = ". $this->db->escape($input->post('photos_per_page')) .",
             archive_per_page = ". $this->db->escape($input->post('archive_per_page')) .",
             resize_to = ". $this->db->escape($input->post('resize_to')) .",
             thumbnail_width = ". $this->db->escape($input->post('thumbnail_width')) .",
             thumbnail_height = ". $this->db->escape($input->post('thumbnail_height')) .",
             comments_allowed = ". $this->db->escape($input->post('comments_allowed')) .",
             dynamic_resize = ". $this->db->escape($input->post('dynamic_resize')) .",
             blog_title = ". $this->db->escape($input->post('blog_title')) .",
             display_menu_photos = ". $this->db->escape($input->post('display_menu_photos')) .",
             display_menu_archives = ". $this->db->escape($input->post('display_menu_archives')) .",
             display_menu_links = ". $this->db->escape($input->post('display_menu_links')) .",
             display_menu_pages = ". $this->db->escape($input->post('display_menu_pages')) .",
             mode = ". $this->db->escape($input->post('mode')) .",
             rss = ". $this->db->escape($input->post('rss')) ."
    ";
    if ( ! $this->db->query($sql))
    {
      return FALSE;
    }
    else
    {
      if ( ! $this->update_flickr_settings($input)) return FALSE;
      return TRUE;
    }
  }
  
  
  
  /**
  * SETTINGS: Update flickr settings
  * 
  * @access public
  * @param  string
  * @return boolean
  */
  function update_flickr_settings($input)
  {
    // flickr settings
    $sql = "
      UPDATE ".$this->prefix."flickr_settings
         SET flickr_user_id = ". $this->db->escape($input->post('flickr_user_id')) .",
             tag = ". $this->db->escape($input->post('tag')) .",
             photoset_id = ". $this->db->escape($input->post('photoset_id')) .",
             display_size = ". $this->db->escape($input->post('display_size')) ."
    ";
    if ( ! $this->db->query($sql))
    {
      return FALSE;
    }
    else
    {
      return TRUE;
    }
  }
  
  
  
  /**
  * SETTINGS: Update photo view
  * 
  * @access public
  * @param  string
  * @return boolean
  */
  function update_photo_view($view)
  {
    $sql = "
      UPDATE ".$this->prefix."settings
         SET photo_view = ". $this->db->escape($view) ."
    ";
    return $this->db->query($sql);
  }
  
  
  /**
  * PAGES: Get all pages
  * 
  * @access public
  * @return object
  */
  function get_all_pages($include_subpages = FALSE)
  {
    if ($include_subpages == FALSE)
    {
      $this->db->where('parent_id', 0);
    }
    return $this->db->get($this->prefix . 'pages');
  }
  
  
  /**
  * PAGES: Get all subpages
  * 
  * @access public
  * @return object
  */
  function get_all_subpages($page_id)
  {
    $this->db->where('parent_id', $page_id);
    return $this->db->get($this->prefix . 'pages');
  }
  
  /**
  * PAGES: Get one page
  * 
  * @access public
  * @param  string
  * @return object
  */
  function get_page($id)
  {
    $this->db->where('id', $id);
    return $this->db->get($this->prefix . 'pages');
  }
  
  
  /**
  * PAGES: Insert new page
  * 
  * @access public
  * @param  object
  * @return boolean
  */
  function insert_page($input)
  {
    $sql = "
      INSERT INTO ".$this->prefix."pages
              SET title = ". $this->db->escape($input->post('title')) .",
                  body = ". $this->db->escape($input->post('body')) .",
                  parent_id = ". $this->db->escape($input->post('parent_id')) ."
    ";
    return $this->db->query($sql);
  }
  
  
  /**
  * PAGES: Update page
  * 
  * @access public
  * @param  string
  * @param  object
  * @return boolean
  */
  function update_page($id, $input)
  {
    $sql = "
      UPDATE ".$this->prefix."pages
         SET title = ". $this->db->escape($input->post('title')) .",
             body = ". $this->db->escape($input->post('body')) .",
             parent_id = ". $this->db->escape($input->post('parent_id')) ."
       WHERE id = ". $this->db->escape($id) ."
    ";
    return $this->db->query($sql);
  }
  
  
  /**
  * GLOBAL: Get main menu
  * 
  * @access public
  * @return array
  */
  function get_main_menu()
  {
    // Get settings
    $settings = $this->get_settings();
    $settings = $settings->row_array();
    
    // Get active link
    $mainpage = $this->uri->segment(1);
    $subpage = $this->uri->segment(2);
    $variable = $this->uri->segment(3);
    
    $photo_class = $archives_class = $links_class = 'mainmenu_inactive';
    
    switch ($mainpage)
    {
      case 'photos':
        $photo_class = 'mainmenu_active';
        break;
      case 'photo':
        $photo_class = 'mainmenu_active';
        break;
      case '':
        $photo_class = 'mainmenu_active';
        break;
      case 'archives':
        $archives_class = 'mainmenu_active';
        break;
      case 'links':
        $links_class = 'mainmenu_active';
        break;
    }
    
    // Defaults
    if ($settings['display_menu_photos'])
    {
      $data[] = array(
        'link_url' => site_url(),
        'link_txt' => 'Photos',
        'link_class' => $photo_class
      );
    }
    
    if ($settings['display_menu_archives'])
    {
      $data[] = array(
        'link_url' => site_url() . '/archives',
        'link_txt' => 'Archives',
        'link_class' => $archives_class
      );
    }
    
    // Links
    if ($this->db->count_all($this->prefix . 'links') && $settings['display_menu_links'])
    {
      $data[] = array(
        'link_url' => site_url() . '/links',
        'link_txt' => 'Links',
        'link_class' => $links_class
      );
    }    

    // Pages
    if ($settings['display_menu_pages'])
    {
      $pages = $this->get_all_pages();
      $parent_page_id = $this->get_parent_page($variable);
    
      foreach($pages->result_array() as $page)
      {
        $page_class = 'mainmenu_inactive';
        if ($mainpage == 'pages' && ($variable == $page['id'] OR $page['id'] == $parent_page_id))
        {
          $page_class = 'mainmenu_active';
        }
        $data[] = array(
          'link_url' => site_url() . '/pages/index/' . $page['id'],
          'link_txt' => $page['title'],
          'link_class' => $page_class
        );
      }
    }
    
    return $data;
  }
  
  
  /**
  * PAGES: get parent page
  * 
  * @access public
  * @return int
  */
  function get_parent_page($page_id)
  {
    // Get menu
    $this->db->where('id', $page_id);
    $query = $this->db->get($this->prefix . 'pages');
    if ($page = $query->row_array())
    {
      return $page['parent_id'];
    }
    return 0;
  }
  
  
  /**
  * GENERAL: get global tags
  * 
  * @access public
  * @return array
  */
  function get_global_tags()
  {
    // Settings
    $settings = $this->get_settings();
    $settings = $settings->row_array();
    
    // Get menu
    $return['main_menu'] = $this->get_main_menu();
    
    // Get settings
    $return['blog_title'] = $settings['blog_title'];
    
    // Site URL
    $return['site_url'] = site_url();
    
    // Charset
    $return['charset'] = $this->config->item('charset');
    
    // RSS link
    if ($settings['rss'])
    {
      switch ($settings['mode'])
      {
        case 'FLICKR':
          $flickr_settings = $this->get_flickr_settings();
          $feed_url = 'http://api.flickr.com/services/feeds/photos_public.gne?id=' . $flickr_settings['flickr_user_id'] . '&amp;lang=en-us&amp;format=rss_200';
          break;
        default:
          $feed_url = site_url('feed');
          break;
      }

      $return['rss'] = '<link rel="alternate" type="application/rss+xml" title="' . $settings['blog_title'] . ' RSS Feed" href="' . $feed_url . '" />';
    }
    else
    {
      $return['rss'] = '';
    }
    
    return $return;
  }
  
  
  /**
  * PHOTO: Get previous photo
  * 
  * @access public
  * @param  string
  * @return object
  */
  function get_prev_photo($start)
  {
    $this->db->order_by('upload_date DESC, id DESC');
    return $this->db->get($this->prefix . 'photos', 1, $start + 1);
  }
  
  
  /**
  * PHOTO: Get next photo
  * 
  * @access public
  * @param  string
  * @return object
  */
  function get_next_photo($start)
  {
    if ($start)
    {
      $this->db->order_by('upload_date DESC, id DESC');
      return $this->db->get($this->prefix . 'photos', 1, $start - 1);
    }
    else
    {
      return $this->db->get($this->prefix . 'photos', 0, 0);
    }
  }
  
  
  /**
  * TAGS: Get all tags for a photo
  * 
  * @access public
  * @param  string
  * @return object
  */
  function get_tags($photo_id)
  {
    $this->db->where('photo_id', $photo_id);
    return $this->db->get($this->prefix . 'tags');
  }
  
  
  /**
  * ARCHIVES: Get archive months 
  * 
  * @access public
  * @return object
  */
  function get_archive_months()
  {
    $sql = "
      SELECT DISTINCT DATE_FORMAT(upload_date, '%Y-%m') as archive_month
        FROM ".$this->prefix."photos
        ORDER BY DATE_FORMAT(upload_date, '%Y. %m') DESC
    ";
    return $this->db->query($sql);
  }
  
  
  /**
  * TAGS: Count by tags
  * 
  * @access public
  * @return object
  */
  function get_tags_count()
  {
    $sql = "
      SELECT tag, COUNT(1) as counter
        FROM ".$this->prefix."tags
       GROUP BY tag
       ORDER BY tag
    ";
    
    return $this->db->query($sql);   
  }
  
  
  /**
  * LINKS: Get all links
  * 
  * @access public
  * @return object
  */
  function get_all_links()
  {
    $this->db->order_by('title');
    return $this->db->get($this->prefix . 'links');
  }
  
  
  /**
  * LINKS: Insert link
  * 
  * @access public
  * @param  object
  * @return boolean
  */
  function insert_link($input)
  {
    $sql = "
      INSERT INTO ".$this->prefix."links
              SET title = ". $this->db->escape($input->post('title')) .",
                  url = ". $this->db->escape($input->post('url')) .",
                  body = ". $this->db->escape($input->post('body')) ."
    ";
    return $this->db->query($sql);
  }
  
  
  /**
  * LINKS: Update link
  * 
  * @access public
  * @param  string
  * @param  object
  * @return boolean
  */
  function update_link($id, $input)
  {
    $sql = "
      UPDATE ".$this->prefix."links
         SET title = ". $this->db->escape($input->post('title')) .",
             url = ". $this->db->escape($input->post('url')) .",
             body = ". $this->db->escape($input->post('body')) ."
       WHERE id = ". $this->db->escape($id) ."
    ";
    return $this->db->query($sql);
  }
  
  
  /**
  * LINKS: Get link
  * 
  * @access public
  * @param  string
  * @return object
  */
  function get_link($id)
  { 
    $this->db->where('id', $id);
    return $this->db->get($this->prefix . 'links');
  }
  
  
  /**
  * COMMENTS: Insert new comment
  * 
  * @access public
  * @param  string
  * @param  string
  * @return boolean
  */
  function insert_comment($input, $photo_id)
   {
    $sql = "
      INSERT INTO ".$this->prefix."comments
         SET photo_id = ". $this->db->escape($photo_id) .",
             name = ". $this->db->escape($input->post('comment_name')) .",
             email = ". $this->db->escape($input->post('comment_email')) .",
             url = '',
             body = ". $this->db->escape($input->post('comment_body')) .",
             post_date = NOW()
    ";
    return $this->db->query($sql);
  }
  
  
  /**
  * Get all comments
  * 
  * @access public
  * @param  string
  * @param  string
  * @param  string
  * @return object
  */
  function get_all_comments($photo_id, $offset, $start)
  {
    $sql = "
      SELECT *
        FROM ".$this->prefix."comments
       WHERE photo_id = " . $this->db->escape($photo_id) . "
       ORDER BY post_date DESC
       LIMIT ". $start .", ". $offset ."
    ";
    return $this->db->query($sql);
  }
  
  
  /**
  * Get last comment
  * 
  * @access public
  * @return string
  */
  function get_last_comment()
  {
    $sql = "
      SELECT c.body,
             p.title,
             p.id
        FROM ".$this->prefix."comments c,
             ".$this->prefix."photos p
       WHERE p.id = c.photo_id
       ORDER BY c.post_date DESC
       LIMIT 1
    ";
    $query = $this->db->query($sql);
    $row = $query->row_array();
    if (isset($row['body']))
    {
      return $row;
    }
    return FALSE;
  }
  
  
  /**
  * SEARCH: Search photos by tagname
  * 
  * @access public
  * @param  string
  * @param  string
  * @param  string
  * @return boolean
  */
  function get_photos_by_tagname($tag, $offset, $start)
  {
    $sql = "
      SELECT DISTINCT p.*                      
                 FROM ".$this->prefix."tags t,
                      ".$this->prefix."photos p
                WHERE t.tag = ". $this->db->escape($tag) ."
                  AND p.id = t.photo_id
                ORDER BY p.upload_date DESC
                LIMIT " . $start . ", " . $offset . "
    ";
    return $this->db->query($sql);
  }
  
  
  /**
  * SEARCH: Count photos by tagname
  * 
  * @access public
  * @param  string
  * @return integer
  */
  function count_photos_by_tagname($tag)
  {
    $sql = "
      SELECT DISTINCT p.*
                 FROM ".$this->prefix."tags t,
                      ".$this->prefix."photos p
                WHERE t.tag = ". $this->db->escape($tag) ."
                  AND p.id = t.photo_id
    ";
    $result = $this->db->query($sql);
    return $result->num_rows();
  }
  
  
  /**
  * ARCHIVES: Get photos by month
  * 
  * @access public
  * @param  string
  * @param  string
  * @param  string
  * @param  string
  * @return object
  */
  function get_photos_by_month($year, $month, $offset, $start)
  {
    $sql = "
      SELECT *
        FROM ".$this->prefix."photos
       WHERE DATE_FORMAT(upload_date, '%Y-%m') = ". $this->db->escape($year . '-' . $month) ."
       LIMIT ". $start . ", " . $offset ."
    ";
    return $this->db->query($sql);
  }
  
  
  /**
  * SEARCH: Count photos by month
  * 
  * @access public
  * @param  string
  * @param  string
  * @return boolean
  */
  function count_photos_by_month($year, $month)
  {
    $sql = "
      SELECT *
        FROM ".$this->prefix."photos
       WHERE DATE_FORMAT(upload_date, '%Y-%m') = ". $this->db->escape($year . '-' . $month) ."
    ";
    $result = $this->db->query($sql);
    return $result->num_rows();
  }
  
  
  /**
  * TAGS: Delete tags
  * 
  * @access public
  * @param  string
  * @return boolean
  */
  function delete_tags($photo_id)
  {
    return $this->db->delete($this->prefix . 'tags', array('photo_id' => $photo_id));
  }
  
  
  /**
  * COMMENTS: Count all comments for a photo
  * 
  * @access public
  * @param  string
  * @return integer
  */
  function count_all_comments($photo_id)
  {
    $this->db->where('photo_id', $photo_id);
    return $this->db->count_all($this->prefix . 'comments');
  }
  
  
  /**
  * GLOBAL: Insert captcha record
  * 
  * @access public
  * @param  array
  * @return boolean
  */
  function insert_captcha($data)
  {
	 $query = $this->db->insert_string($this->prefix . 'captcha', $data);
	 return $this->db->query($query);
  }
  
  
  /**
  * GLOBAL: Clean expired captchas 
  * 
  * @access public
  * @param  string
  * @return boolean
  */
  function delete_old_captchas($expiration)
  {
    return $this->db->query("DELETE FROM ".$this->prefix."captcha WHERE captcha_time < ".$expiration);		
  }
  
  
  /**
  * GLOBAL: Get captcha
  * 
  * @access public
  * @param  object
  * @return boolean
  */
  function get_captcha($input)
  {
    $sql = "
      SELECT COUNT(1) AS count
        FROM ".$this->prefix."captcha 
       WHERE word = ". $this->db->escape($input['word']) ."
         AND ip_address = ". $this->db->escape($input['ip_address']) ."
         AND captcha_time > ". $this->db->escape($input['expiration']) ."
    ";
    $query = $this->db->query($sql);
  	$row = $query->row();
  
  	if ($row->count == 0)
  	{
  		return FALSE;
  	}
  	else
  	{
  	 return TRUE;
  	}
  }
  
  
  /**
  * COMMENT: Delete comment
  * 
  * @access public
  * @param  string
  * @return boolean
  */
  function delete_comment($id)
  {
    return $this->db->delete($this->prefix . 'comments', array('id' => $id));
  }
  
  
  /**
  * COMMENT: Delete comments for photo
  * 
  * @access public
  * @param  string
  * @return boolean
  */
  function delete_comments_for_photo($photo_id)
  {
    return $this->db->delete($this->prefix . 'comments', array('photo_id' => $photo_id));
  }
  
  
  /**
  * PAGE: Delete page
  * 
  * @access public
  * @param  string
  * @return boolean
  */
  function delete_page($id)
  {
    $this->db->delete($this->prefix . 'pages', array('parent_id' => $id));
    return $this->db->delete($this->prefix . 'pages', array('id' => $id));
  }
  
  
  /**
  * LINK: Delete link
  * 
  * @access public
  * @param  string
  * @return boolean
  */
  function delete_link($id)
  {
    return $this->db->delete($this->prefix . 'links', array('id' => $id));
  }
  
  
  /**
  * Gets flickr settings
  */
  function get_flickr_settings()
  {
    $query = $this->db->get($this->prefix . 'flickr_settings');
    
    if ($query->num_rows())
    {
      return $query->row_array();
    }
    else
    {
      // Insert default settings to flickr settings
      $defaults = array(
        'flickr_user_id' => '',
        'tag' => '',
        'photoset_id' => '',
        'display_size' => ''
      );
      
      $this->db->insert($this->prefix . 'flickr_settings', $defaults);
      
      $query = $this->db->get($this->prefix . 'flickr_settings');
      return $query->row_array();
    }
  }
  
  
  /**
  * Gets latest feed
  */
  function get_recent_posts()
  {
    $this->db->order_by('upload_date', 'desc');
    $this->db->limit(10);
    return $this->db->get($this->prefix . 'photos');
  }
}
?>